__all__ = [
    "BasicProperty",
    "BasicPropertySet",
    "Property",
    "PropertySet",
    "PropertyValue",
]

from typing import Any, Iterable, List, TypeVar, Union

from java.beans import PropertyChangeListener
from java.lang import Class, Object, String

T = TypeVar("T")


class Property(object):
    def getDefaultValue(self):
        # type: () -> Any
        raise NotImplementedError

    def getName(self):
        # type: () -> String
        raise NotImplementedError

    def getType(self):
        # type: () -> Class
        raise NotImplementedError


class PropertySet(object):
    @staticmethod
    def builder():
        # type: () -> BasicPropertySet.Builder
        raise NotImplementedError

    def extend(self, parent):
        # type: (PropertySet) -> PropertySet
        raise NotImplementedError

    def isExtended(self, prop):
        # type: (Property) -> bool
        raise NotImplementedError

    def newDefaultInstance(self):
        # type: () -> PropertySet
        raise NotImplementedError

    def newExtension(self):
        # type: () -> PropertySet
        raise NotImplementedError


class BasicProperty(Property, Object):
    _name = None  # type: String
    _clazz = None  # type: Any
    _defaultValue = None  # type: Any
    _hcode = None  # type: int

    def __init__(self, *args):
        # type: (Any) -> None
        if not args:
            self._hcode = 0
        elif len(args) == 2:
            self._name = args[0]
            self._clazz = args[1]
        elif len(args) == 3:
            self._name = args[0]
            self._clazz = args[1]
            self._defaultValue = args[2]
        else:
            raise NotImplementedError

    def getClazz(self):
        # type: () -> Class
        pass

    def getDefaultValue(self):
        # type: () -> Any
        return self._defaultValue

    def getName(self):
        # type: () -> String
        return self._name

    def getType(self):
        # type: () -> Class
        pass

    def setClazz(self, clazz):
        # type: (Class) -> None
        pass

    def setClazz_(self, clazz):
        # type: (Class) -> BasicProperty
        pass

    def setDefaultValue(self, defaultValue):
        # type: (Any) -> None
        pass

    def setDefaultValue_(self, defaultValue):
        # type: (Any) -> BasicProperty
        pass

    def setName(self, name):
        # type: (String) -> None
        pass

    def setName_(self, name):
        # type: (String) -> BasicProperty
        pass


class BasicPropertySet(Object):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def addPropertySet(self, *args):
        # type: (Any) -> None
        pass

    def contains(self, prop):
        # type: (Property) -> bool
        pass

    def get(self, prop):
        # type: (Property) -> T
        pass

    def getCount(self):
        # type: () -> int
        pass

    def getExtension(self):
        # type: () -> PropertySet
        pass

    def getOrDefault(self, prop):
        # type: (Property) -> T
        pass

    def getOrElse(self, prop, value):
        # type: (Property, T) -> T
        pass

    def getProperties(self):
        # type: () -> List[Property]
        pass

    def getRawValueMap(self):
        # type: () -> Any
        pass

    def getValues(self):
        # type: () -> List[PropertyValue]
        pass

    def isExtended(self, prop):
        # type: (Property) -> bool
        pass

    def isInherited(self, prop):
        # type: (Property) -> bool
        pass

    def iterator(self):
        # type: () -> Iterable[PropertyValue]
        pass

    @staticmethod
    def of(*args):
        # type: (PropertyValue) -> PropertySet
        pass

    def remove(self, prop):
        # type: (Property) -> None
        pass

    def removePropertyChangeListener(self, listener):
        # type: (PropertyChangeListener) -> None
        pass

    def set(self, prop, value):
        # type: (Union[Property, PropertyValue], T) -> None
        pass

    def setDirect(self, prop, value):
        # type: (Property, Object) -> None
        pass

    def setRawValueMap(self, copy):
        # type: (Any) -> None
        pass

    class Builder(Object):
        def build(self):
            # type: () -> BasicPropertySet
            pass

        def set(self, prop, value):
            # type: (Property, T) -> BasicPropertySet.Builder
            pass


class PropertyValue(Object):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def getProperty(self):
        # type: () -> Property
        pass

    def getValue(self):
        # type: () -> Object
        pass

    @staticmethod
    def of(prop, value):
        # type: (Property, Object) -> PropertyValue
        pass



from __future__ import print_function

from typing import Any, Dict, List, Optional, Set, Tuple

from com.inductiveautomation.ignition.common import JsonPath
from com.inductiveautomation.ignition.common.gson import JsonArray, JsonElement
from java.io import InputStream
from java.lang import Enum, Object, String
from java.util.regex import Pattern


class JsonValidator(object):
    AT_ROOT = None  # type: String

    def validate(self, *args):
        # type: (Any) -> Set[ValidationMessage]
        raise NotImplementedError


class BaseJsonValidator(Object, JsonValidator):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def asInt(self, element):
        # type: (JsonElement) -> int
        pass

    def validate(self, *args):
        # type: (Any) -> Set[ValidationMessage]
        pass


class ItemsValidator(BaseJsonValidator):
    PROPERTY = None  # type: String

    def __init__(self, *args):
        # type: (Any) -> None
        super(ItemsValidator, self).__init__(*args)

    def deriveDefaultArray(self, includeExample=None):
        # type: (Optional[bool]) -> JsonArray
        pass

    def findSchemaForIndex(self, index):
        # type: (int) -> Optional[JsonSchema]
        pass

    def getDefaultItem(
        self,
    ):
        # type: () -> Optional[JsonElement]
        pass

    def getSchemas(self):
        # type: () -> List[JsonSchema]
        pass

    def isBoundedSchema(self):
        # type: () -> Set[ValidationMessage]
        pass


class JsonSchema(BaseJsonValidator):
    IGNITION_SCHEMA_PATTERN = None  # type: Pattern
    IGNITION_SCHEMA_URN = None  # type: String

    def __init__(self, *args):
        # type: (Any) -> None
        super(JsonSchema, self).__init__(*args)

    def acceptsType(self, type):
        # type: (JsonType) -> bool
        pass

    def findAncestor(self):
        # type: () -> JsonSchema
        pass

    def findAncestorMatchingPath(self, matchingPath):
        # type: (String) -> Optional[JsonSchema]
        pass

    def findSchemaForPath(self, path):
        # type: (JsonPath) -> Optional[JsonSchema]
        pass

    def getChildDefaultValue(self, type):
        # type: (JsonType) -> Optional[JsonElement]
        pass

    def getDeclaredProperties(self):
        # type: () -> List[String]
        pass

    def getDefaultValue(self, includeExamples=None):
        # type: (Optional[bool]) -> JsonElement
        pass

    def getDescription(self):
        # type: () -> String
        pass

    def getDynamicSuggestionPath(self):
        # type: () -> String
        pass

    def getEnumChoices(self):
        # type: () -> Dict[String, JsonElement]
        pass

    def getExamples(self):
        # type: () -> List[JsonElement]
        pass

    def getExampleValue(self):
        # type: () -> Optional[JsonElement]
        pass

    def getExtension(self, key):
        # type: (String) -> Optional[JsonElement]
        pass

    def getFormat(self):
        # type: () -> String
        pass

    def getItemsValidator(self):
        # type: () -> ItemsValidator
        pass

    def getRefSchemaNode(self, String=None):
        # type: (String) -> JsonElement
        pass

    def getSchemasForItems(self):
        # type: () -> List[JsonSchema]
        pass

    def getSchemasForProperties(self):
        # type: () -> Dict[String, JsonSchema]
        pass

    def getSuggestions(self):
        # type: () -> Dict[String, JsonElement]
        pass

    def getTitle(self):
        # type: () -> String
        pass

    def getTypeValidator(self):
        # type: () -> TypeValidator
        pass

    def getVisibleWhenCondition(self):
        # type: () -> Optional[Tuple[String, List[JsonElement]]]
        pass

    def hasChildren(self):
        # type: () -> bool
        pass

    def isDeprecated(self):
        # type: () -> bool
        pass

    def isType(self, type):
        # type: (JsonType) -> bool
        pass

    @staticmethod
    def parse(stream):
        # type: (InputStream) -> JsonSchema
        pass


class JsonType(Enum):
    OBJECT = "object"
    ARRAY = "array"
    STRING = "string"
    NUMBER = "number"
    INTEGER = "integer"
    BOOLEAN = "boolean"
    NULL = "null"
    DATASET = "dataset"
    DATE = "date"
    UNKNOWN = "unknown"

    @staticmethod
    def typeOf(element):
        # type: (JsonElement) -> JsonType
        pass

    @staticmethod
    def values():
        # type: () -> List[JsonType]
        pass


class ValidationMessage(Object):
    def getCode(self):
        # type: () -> String
        pass

    def getMessage(self):
        # type: () -> String
        pass

    def getPath(self):
        # type: () -> String
        pass

    def getType(self):
        # type: () -> String
        pass

    def setArguments(self, *args):
        # type: (String) -> None
        pass

    def setType(self, type):
        # type: (String) -> None
        pass

    class Builder(Object):
        def arguments(self, *args):
            # type: (String) -> ValidationMessage.Builder
            pass

        def build(self):
            # type: () -> ValidationMessage
            pass

        def code(self, code):
            # type: (String) -> ValidationMessage.Builder
            pass

        def format(self, format):
            # type: (String) -> ValidationMessage.Builder
            pass

        def path(self, path):
            # type: (String) -> ValidationMessage.Builder
            pass

        def type(self, type):
            # type: (String) -> ValidationMessage.Builder
            pass


class TypeValidator(BaseJsonValidator):
    def __init__(self, *args):
        # type: (Any) -> None
        super(TypeValidator, self).__init__(*args)

    def getAcceptedTypes(self):
        # type: () -> Set[JsonType]
        pass



__all__ = ["Request", "RequestWatcher"]


class RequestWatcher(object):
    def block(self):
        raise NotImplementedError

    def compose(self, requestWatchers):
        raise NotImplementedError


class Request(RequestWatcher):
    def block(self):
        pass

    def cancel(self):
        raise NotImplementedError

    def compose(self, requestWatchers):
        pass

    def get(self):
        raise NotImplementedError

    def getError(self):
        raise NotImplementedError

    def onError(self, func):
        raise NotImplementedError

    def onSuccess(self, func):
        raise NotImplementedError



from __future__ import print_function

__all__ = [
    "Color",
    "Component",
    "Container",
    "Frame",
    "Image",
    "Toolkit",
    "Window",
]

from typing import Any, Optional

from java.lang import Object


class Color(Object):
    def __init__(self, *args):
        # type: (Any) -> None
        print(args)


class Component(Object):
    def imageUpdate(
        self,
        img,  # type: Image
        infoflags,  # type: int
        x,  # type: int
        y,  # type: int
        width,  # type: int
        height,  # type: int
    ):
        # type: (...) -> bool
        pass


class Container(Component):
    def add(self, *args):
        # type: (Any) -> Optional[Component]
        pass


class Image(Object):
    def flush(self):
        # type: () -> None
        pass

    def getAccelerationPriority(self):
        # type: () -> float
        pass

    def setAccelerationPriority(self, priority):
        # type: (float) -> None
        pass


class Toolkit(Object):
    def __init__(self):
        # type: () -> None
        pass

    def beep(self):
        # type: () -> None
        print(self, "Beep!")

    @staticmethod
    def getDefaultToolkit():
        # type: () -> Toolkit
        return Toolkit()


class Window(Container):
    def __init__(self, *args):
        # type: (Any) -> None
        pass


class Frame(Window):
    def __init__(self, *args):
        # type: (Any) -> None
        super(Frame, self).__init__(*args)



"""SFC Functions.

The following functions give you access to interact with the SFCs in the
Gateway.
"""

from __future__ import print_function

__all__ = [
    "cancelChart",
    "getRunningCharts",
    "getVariables",
    "pauseChart",
    "redundantCheckpoint",
    "resumeChart",
    "setVariable",
    "setVariables",
    "startChart",
]

from typing import Any, Dict, Optional

from com.inductiveautomation.ignition.common import BasicDataset
from com.inductiveautomation.sfc.api import PyChartScope
from java.lang import String


def cancelChart(instanceId):
    # type: (String) -> None
    """Cancels the execution of a running chart instance.

    Any running steps will be told to stop, and the chart will enter
    Canceling state.

    Args:
        instanceId: The ID of the chart instance to cancel.

    Raises:
        KeyError: If the ID does not match any running chart instance.
    """
    if not instanceId:
        raise KeyError("Invalid UUID string: {}".format(instanceId))


def getRunningCharts(chartPath=None):
    # type: (Optional[String]) -> BasicDataset
    """Retrieves information about running charts.

    Can search all running charts, or be filtered charts at a specific
    path. This function will return charts that are in a Paused state.

    Args:
        chartPath: The path to a chart to filter on: i.e.,
            "folder/chartName". If specified, only charts at the path
            will be included in the returned dataset. If omitted, the
            function will return data for all active charts.

    Returns:
        A dataset with information on the active chart.
    """
    print(chartPath)
    return BasicDataset()


def getVariables(instanceId):
    # type: (String) -> PyChartScope
    """Get the variables in a chart instance's scope.

    Commonly used to check the value of a Chart Parameter, or determine
    how long the chart has been running for.

    Args:
        instanceId: The instance identifier of the chart.

    Returns:
        A python dictionary of variables. Step scopes for active steps
        are found under the "activeSteps" key.
    """
    print(instanceId)
    return PyChartScope()


def pauseChart(instanceId):
    # type: (String) -> None
    """Pauses a running chart instance.

    Any running steps will be told to pause, and the chart will enter
    Pausing state.

    Args:
        instanceId: The ID of the chart instance to pause.

    Raises:
        KeyError: If the ID does not match any running chart instance.
    """
    if not instanceId:
        raise KeyError("Invalid UUID string: {}".format(instanceId))


def redundantCheckpoint(instanceId):
    # type: (String) -> None
    """Synchronizes chart and step variables of the specified chart
    instance across a redundant cluster, allowing the chart instance to
    continue where it left off if a redundant failover occurs.

    Args:
        instanceId: The instance identifier of the chart.
    """
    print(instanceId)


def resumeChart(instanceId):
    # type: (String) -> None
    """Resumes a chart that was paused.

    Steps which were previously paused will be resumed, and chart will
    enter Resuming state.

    Args:
        instanceId: The ID of the chart instance to resume.

    Raises:
        KeyError: If the ID does not match any running chart instance.
    """
    if not instanceId:
        raise KeyError("Invalid UUID string: {}".format(instanceId))


def setVariable(instanceId, stepId, variableName, variableValue):
    # type: (String, String, String, Any) -> None
    """Sets a variable inside a currently running chart.

    Args:
        instanceId: The instance identifier of the chart.
        stepId: The id for a step inside of a chart. If omitted the
            function will target a chart scoped variable.
        variableName: The name of the variable to set.
        variableValue: The value for the variable to be set to.
    """
    print(instanceId, stepId, variableName, variableValue)


def setVariables(instanceId, stepId, variableMap):
    # type: (String, String, Dict[String, Any]) -> None
    """Sets any number of variables inside a currently running chart.

    Args:
        instanceId: The instance identifier of the chart.
        stepId: The id for a step inside of a chart. If omitted
            the function will target a chart scoped variable.
        variableMap: A dictionary containing the name:value pairs
            of the variables to set.
    """
    print(instanceId, stepId, variableMap)


def startChart(projectName, chartPath, arguments):
    # type: (String, String, Dict[String, Any]) -> String
    """Starts a new instance of a chart.

    The chart must be set to "Callable" execution mode.

    Args:
        projectName: The name of the project that the chart was created
            in.
        chartPath: The path to the chart, for example
            "ChartFolder/ChartName"
        arguments: A dictionary containing the name:value pairs of the
            variables to set.

    Returns:
        The unique ID of this chart.
    """
    print(projectName, chartPath, arguments)
    return "UUID"



"""Security Functions.

The following functions give you access to interact with the users and
roles in the Gateway. These functions require the Vision module, as
these functions can only be used with User Sources and their interaction
with Vision Clients.
"""

from __future__ import print_function

__all__ = [
    "getRoles",
    "getUserRoles",
    "getUsername",
    "isScreenLocked",
    "lockScreen",
    "logout",
    "switchUser",
    "unlockScreen",
    "validateUser",
]

import getpass

from typing import Optional, Tuple

from java.lang import String
from java.util import EventObject


def getRoles():
    # type: () -> Tuple[String, ...]
    """Finds the roles that the currently logged in user has, returns
    them as a Python tuple of strings.

    Returns:
        A list of the roles (strings) that are assigned to the current
        user.
    """
    return "Administrator", "Developer"


def getUserRoles(
    username,  # type: String
    password,  # type: String
    authProfile="",  # type: String
    timeout=60000,  # type: int
):
    # type: (...) -> Optional[Tuple[String, ...]]
    """Fetches the roles for a user from the Gateway.

    This may not be the currently logged in user. Requires the password
    for that user. If the authentication profile name is omitted, then
    the current project's default authentication profile is used.

    Args:
        username: The username to fetch roles for.
        password: The password for the user.
        authProfile: The name of the authentication profile to run
            against. Leaving this out will use the project's default
            profile. Optional.
        timeout: Timeout for Client-to-Gateway communication. Default is
            60,000ms. Optional.


    Returns:
        A list of the roles that this user has, if the user
        authenticates successfully. Otherwise, returns None.
    """
    print(username, password, authProfile, timeout)
    return "Administrator", "Developer"


def getUsername():
    # type: () -> String
    """Returns the currently logged-in username.

    Returns:
        The current username.
    """
    return getpass.getuser()


def isScreenLocked():
    # type: () -> bool
    """Returns whether or not the screen is currently locked.

    Returns:
        A flag indication whether or not the screen is currently locked.
    """
    return False


def lockScreen(obscure=False):
    # type: (Optional[bool]) -> None
    """Used to put a running Client in lock-screen mode.

    The screen can be unlocked by the user with the proper credentials,
    or by scripting via the system.security.unlockScreen() function.

    Args:
        obscure: If True, the locked screen will be opaque, otherwise
            it will be partially visible. Optional.
    """
    print(obscure)


def logout():
    # type: () -> None
    """Logs out of the Client for the current user and brings the Client
    to the login screen.
    """
    pass


def switchUser(username, password, event, hideError=False):
    # type: (String, String, EventObject, Optional[bool]) -> bool
    """Attempts to switch the current user on the fly.

    If the given username and password fail, this function will return
    False. If it succeeds, then all currently opened windows are closed,
    the user is switched, and windows are then re-opened in the states
    that they were in.

    If an event object is passed to this function, the parent window of
    the event object will not be re-opened after a successful user
    switch. This is to support the common case of having a switch-user
    screen that you want to disappear after the switch takes place.

    Args:
        username: The username to try and switch to.
        password: The password to authenticate with.
        event: If specified, the enclosing window for this event's
            component will be closed in the switch user process.
        hideError: If True, no error will be shown if the switch
            user function fails. Default is False. Optional.

    Returns:
        False if the switch user operation failed, True
        otherwise.
    """
    print(username, password, event, hideError)
    return True


def unlockScreen():
    # type: () -> None
    """Unlocks the Client, if it is currently in lock-screen mode."""
    pass


def validateUser(username, password, authProfile="", timeout=60000):
    # type: (String, String, Optional[String], Optional[int]) -> bool
    """Tests credentials (username and password) against an
    authentication profile.

    Returns a boolean based upon whether or not the authentication
    profile accepts the credentials. If the authentication profile name
    is omitted, then the current project's default authentication
    profile is used.

    Args:
        username: The username to validate.
        password: The password for the user.
        authProfile: The name of the authentication profile to run
            against. Leaving this out will use the project's default
            profile. Optional.
        timeout: Timeout for Client-to-Gateway communication. Default is
            60,000ms. Optional.

    Returns:
        False if the user failed to authenticate, True if the
        username/password was a valid combination.
    """
    print(username, password, authProfile, timeout)
    return True



"""DNP3 Functions.

The following functions give you access to interact with the DNP3
devices.
"""

from __future__ import print_function

__all__ = [
    "CLOSE",
    "LATCH_OFF",
    "LATCH_ON",
    "NUL",
    "PULSE_OFF",
    "PULSE_ON",
    "TRIP",
    "directOperateAnalog",
    "directOperateBinary",
    "freezeAnalogs",
    "freezeAnalogsAtTime",
    "freezeCounters",
    "freezeCountersAtTime",
    "selectOperateAnalog",
    "selectOperateBinary",
]

from typing import List, Optional, Union

# Constants
from java.lang import String

NUL = 0
PULSE_ON = 1
PULSE_OFF = 2
LATCH_ON = 3
LATCH_OFF = 4
CLOSE = 1
TRIP = 2


def directOperateAnalog(
    deviceName,  # type: String
    index,  # type: int
    value,  # type: Union[float, int]
    variation=None,  # type: Optional[int]
):
    # type: (...) -> int
    """Issues a Select-And-Operate command to set an analog value in an
    analog output point.

    Args:
        deviceName: The name of the DNP3 device driver.
        index: The index of the object to be modified in the outstation.
        value: The analog value that is requested (of type int, short,
            float, or double).
        variation: The DNP3 object variation to use in the request.
            Optional.

    Returns:
        The DNP3 status code of the response, as an integer.
    """
    print(deviceName, index, value, variation)
    return 0


def directOperateBinary(
    deviceName,  # type: String
    indexes,  # type: List[int]
    opType,  # type: int
    tcCode=None,  # type: Optional[int]
    count=None,  # type: Optional[int]
    onTime=None,  # type: Optional[int]
    offTime=None,  # type: Optional[int]
):
    # type: (...) -> int
    """Issues a Direct-Operate command for digital control operations
    at binary output points (CROB).

    Args:
        deviceName: The name of the DNP3 device driver.
        indexes: A list of indexes of the objects to be modified in the
            outstation.
        opType: The type of the operation. 0=NUL, 1=PULSE_ON,
            2=PULSE_OFF, 3=LATCH_ON, 4=LATCH_OFF.
        tcCode: The Trip-Close code, used in conjunction with the
            opType. 0=NUL, 1=CLOSE, 2=TRIP. Optional.
        count: The number of times the outstation shall execute the
            operation. Optional.
        onTime: The duration that the output drive remains active, in
            millis. Optional.
        offTime: The duration that the output drive remains non-active,
            in millis. Optional.

    Returns:
        The DNP3 status code of the response, as an integer.
    """
    print(deviceName, indexes, opType, tcCode, count, onTime, offTime)
    return 0


def freezeAnalogs(deviceName, indexes):
    # type: (String, List[int]) -> None
    """Issues a freeze command on the given analog outputs.

    Args:
        deviceName: The name of the DNP3 device driver.
        indexes: An optional list of specific indexes on which to issue
            the freeze command. An empty list can be passed to freeze
            all analogs.
    """
    print(deviceName, indexes)


def freezeAnalogsAtTime(deviceName, absoluteTime, intervalTime, indexes):
    # type: (String, int, int, List[int]) -> None
    """Issues a freeze command on the given analog outputs at the given
    time for the specified duration.

    Args:
        deviceName: The name of the DNP3 device driver.
        absoluteTime: The absolute time at which to freeze, in millis.
        intervalTime: The interval at which to periodically freeze, in
            millis.
        indexes: An optional list of specific indexes on which to issue
            the freeze command. An empty list will freeze all points.
    """
    print(deviceName, absoluteTime, intervalTime, indexes)


def freezeCounters(deviceName, indexes):
    # type: (String, List[int]) -> None
    """Issues a freeze command on the given counters.

    Args:
        deviceName: The name of the DNP3 device driver.
        indexes: An optional list of specific indexes on which to issue
            the freeze command. An empty list can be passed to freeze
            all counters.
    """
    print(deviceName, indexes)


def freezeCountersAtTime(deviceName, absoluteTime, intervalTime, indexes):
    # type: (String, int, int, List[int]) -> None
    """Issues a freeze command on the given counters at the given time
    for the specified duration.

    Args:
        deviceName: The name of the DNP3 device driver.
        absoluteTime: The absolute time at which to freeze, in millis.
        intervalTime: The interval at which to periodically freeze, in
            millis.
        indexes: An optional list of specific indexes on which to issue
            the freeze command. An empty list will freeze all counters.
    """
    print(deviceName, absoluteTime, intervalTime, indexes)


def selectOperateAnalog(
    deviceName,  # type: String
    index,  # type: int
    value,  # type: Union[float, int]
    variation=None,  # type: Optional[int]
):
    # type: (...) -> int
    """Issues a Select-And-Operate command to set an analog value in an
    analog output point.

    Args:
        deviceName: The name of the DNP3 device driver.
        index: The index of the object to be modified in the outstation.
        value: The analog value that is requested (of type int, short,
            float, or double).
        variation: The DNP3 object variation to use in the request.
            Optional.

    Returns:
        The DNP3 status code of the response, as an integer.
    """
    print(deviceName, index, value, variation)
    return 0


def selectOperateBinary(
    deviceName,  # type: String
    indexes,  # type: List[int]
    opType,  # type: int
    tcCode=None,  # type: Optional[int]
    count=None,  # type: Optional[int]
    onTime=None,  # type: Optional[int]
    offTime=None,  # type: Optional[int]
):
    # type: (...) -> int
    """Issues a Select-And-Operate command for digital control
    operations at binary output points (CROB).

    Args:
        deviceName: The name of the DNP3 device driver.
        indexes: A list of indexes of the objects to be modified in the
            outstation.
        opType: The type of operation. 0=NUL, 1=PULSE_ON, 2=PULSE_OFF,
            3=LATCH_ON, 4=LATCH_OFF.
        tcCode: The Trip-Close code, used in conjunction with the
            opType. 0=NUL, 1=CLOSE, 2=TRIP. Optional.
        count: The number of times the outstation shall execute the
            operation. Optional.
        onTime: The duration that the output drive remains active, in
            millis. Optional.
        offTime: The duration that the output drive remains non-active,
            in millis. Optional.

    Returns:
        The DNP3 status code of the response, as an integer.
    """
    print(deviceName, indexes, opType, tcCode, count, onTime, offTime)
    return 0



"""Contains the collections framework, legacy collection classes, event
model, date and time facilities, internationalization, and miscellaneous
utility classes (a string tokenizer, a random-number generator, and a
bit array).
"""

from __future__ import print_function

__all__ = ["Comparator", "Date", "EventObject", "Locale", "UUID"]

from typing import Any, Optional, TypeVar

from dev.thecesrom.utils.decorators import classproperty
from java.lang import Object, String
from java.util.function import Function, ToDoubleFunction, ToIntFunction, ToLongFunction

T = TypeVar("T")


class Comparator(object):
    def compare(self, o1, o2):
        # type: (T, T) -> int
        raise NotImplementedError

    @staticmethod
    def comparing(keyExtractor, keyComparator):
        # type: (Function, Comparator) -> Comparator
        pass

    @staticmethod
    def comparingDouble(keyExtractor):
        # type: (ToDoubleFunction) -> Comparator
        pass

    @staticmethod
    def comparingInt(keyExtractor):
        # type: (ToIntFunction) -> Comparator
        pass

    @staticmethod
    def comparingLong(keyExtractor):
        # type: (ToLongFunction) -> Comparator
        pass

    def equals(self, obj):
        # type: (Object) -> bool
        raise NotImplementedError

    @staticmethod
    def naturalOrder():
        # type: () -> Comparator
        pass

    @staticmethod
    def nullsFirst(comparator):
        # type: (Comparator) -> Comparator
        pass

    @staticmethod
    def nullsLast(comparator):
        # type: (Comparator) -> Comparator
        pass

    def reversed(self):
        # type: () -> Comparator
        pass

    @staticmethod
    def reverseOrder():
        # type: () -> Comparator
        pass

    def thenComparing(self, *args):
        # type: (Any) -> Comparator
        pass

    def thenComparingDouble(self, keyExtractor):
        # type: (ToDoubleFunction) -> Comparator
        pass

    def thenComparingInt(self, keyExtractor):
        # type: (ToIntFunction) -> Comparator
        pass

    def thenComparingLong(self, keyExtractor):
        # type: (ToLongFunction) -> Comparator
        pass


class Date(Object):
    """The class Date represents a specific instant in time, with
    millisecond precision.
    """

    def __init__(self, date=None):
        # type: (Optional[long]) -> None
        print(self, date)

    def after(self, when):
        # type: (Date) -> bool
        pass

    def before(self, when):
        # type: (Date) -> bool
        pass

    def compareTo(self, anotherDate):
        # type: (Date) -> int
        pass

    def getTime(self):
        # type: () -> long
        pass

    def setTime(self, time):
        # type: (long) -> None
        pass


class EventObject(Object):
    """The root class from which all event state objects shall be
    derived.

    All Events are constructed with a reference to the object, the
    "source", that is logically deemed to be the object upon which the
    Event in question initially occurred upon.
    """

    source = None  # type: Object

    def __init__(self, source):
        # type: (Object) -> None
        self.source = source

    def getSource(self):
        return self.source


class Locale(Object):
    """A Locale object represents a specific geographical, political, or
    cultural region. An operation that requires a Locale to perform its
    task is called locale-sensitive and uses the Locale to tailor
    information for the user. For example, displaying a number is a
    locale-sensitive operation; the number should be formatted according
    to the customs and conventions of the user's native country, region,
    or culture.
    """

    country = None  # type: Optional[str]
    language = None  # type: str
    variant = None  # type: Optional[str]

    def __init__(self, language, country=None, variant=None):
        # type: (str, Optional[str], Optional[str]) -> None
        self.language = language
        self.country = country
        self.variant = variant

    def __repr__(self):
        return "{!r}".format(self.__str__())

    def __str__(self):
        ret = self.language
        if self.country:
            ret += "_{}".format(self.country)
        if self.variant:
            ret += "_{}".format(self.variant)
        return unicode(ret)

    @classproperty
    def CANADA(self):
        # type: () -> Locale
        return Locale("en", "CA")

    @classproperty
    def CANADA_FRENCH(self):
        # type: () -> Locale
        return Locale("fr", "CA")

    @classproperty
    def CHINA(self):
        # type: () -> Locale
        return Locale("zh", "CN")

    @classproperty
    def CHINESE(self):
        # type: () -> Locale
        return Locale("zh")

    @classproperty
    def ENGLISH(self):
        # type: () -> Locale
        return Locale("en")

    @classproperty
    def FRANCE(self):
        # type: () -> Locale
        return Locale("fr", "FR")

    @classproperty
    def FRENCH(self):
        # type: () -> Locale
        return Locale("fr")

    @classproperty
    def GERMAN(self):
        # type: () -> Locale
        return Locale("de")

    @classproperty
    def GERMANY(self):
        # type: () -> Locale
        return Locale("de", "DE")

    @classproperty
    def ITALIAN(self):
        # type: () -> Locale
        return Locale("it")

    @classproperty
    def ITALY(self):
        # type: () -> Locale
        return Locale("it", "IT")

    @classproperty
    def JAPAN(self):
        # type: () -> Locale
        return Locale("ja", "JP")

    @classproperty
    def JAPANESE(self):
        # type: () -> Locale
        return Locale("ja")

    @classproperty
    def KOREA(self):
        # type: () -> Locale
        return Locale("ko", "KR")

    @classproperty
    def KOREAN(self):
        # type: () -> Locale
        return Locale("ko")

    @classproperty
    def PRC(self):
        # type: () -> Locale
        return Locale("zh", "CN")

    @classproperty
    def SIMPLIFIED_CHINESE(self):
        # type: () -> Locale
        return Locale("zh", "CN")

    @classproperty
    def TAIWAN(self):
        # type: () -> Locale
        return Locale("zh", "TW")

    @classproperty
    def TRADITIONAL_CHINESE(self):
        # type: () -> Locale
        return Locale("zh", "TW")

    @classproperty
    def UK(self):
        # type: () -> Locale
        return Locale("en", "GB")

    @classproperty
    def US(self):
        # type: () -> Locale
        return Locale("en", "US")


class UUID(Object):
    def __init__(self, mostSigBits, leastSigBits):
        # type: (long, long) -> None
        self._leastSigbits = leastSigBits
        self._mostSigBits = mostSigBits

    def clockSequence(self):
        # type: () -> int
        pass

    def compareTo(self, val):
        # type: (UUID) -> int
        pass

    @staticmethod
    def fromString(name):
        # type: (String) -> UUID
        pass

    def getLeastSignificantBits(self):
        # type: () -> long
        return self._leastSigbits

    def getMostSignificantBits(self):
        # type: () -> long
        return self._mostSigBits

    @staticmethod
    def nameUUIDFromBytes(name):
        # type: (bytearray) -> UUID
        pass

    def node(self):
        # type: () -> long
        pass

    @staticmethod
    def randomUUID():
        # type: () -> UUID
        pass

    def timestamp(self):
        # type: () -> long
        pass

    def variant(self):
        # type: () -> int
        pass

    def version(self):
        # type: () -> int
        pass



"""OPC HDA Functions.

The following functions give you access to interact with the HDA types
of OPC servers.
"""

from __future__ import print_function

__all__ = [
    "browse",
    "getAggregates",
    "getAttributes",
    "getServers",
    "insert",
    "insertReplace",
    "isServerAvailable",
    "readAttributes",
    "readProcessed",
    "readRaw",
    "replace",
]

from typing import Any, List

from com.inductiveautomation.ignition.common.browsing import Results
from com.inductiveautomation.ignition.common.model.values import QualityCode
from com.inductiveautomation.ignition.common.sqltags.history import AggregateInfo
from java.lang import String
from java.util import Date


def browse(root):
    # type: (String) -> List[Results]
    """Performs a browse at the given root.

    Args:
        root: The root at which to browse. Needs to be a qualified path.

    Returns:
        The results of the browse operation from the given root.
    """
    print(root)
    return [Results()]


def getAggregates(serverName):
    # type: (String) -> List[AggregateInfo]
    """Will query the Server for aggregates that it supports.

    Args:
        serverName: The name of the defined OPC-HDA Server to query.

    Returns:
        A list of supported Aggregate objects. Each object has 'id',
        'name', and 'desc' properties defined.
    """
    print(serverName)
    return [AggregateInfo()]


def getAttributes(serverName):
    # type: (String) -> List[AggregateInfo]
    """Queries the given Server for the item attributes that are
    available with system.opchda.readAttributes().

    Args:
        serverName: The name of the defined OPC-HDA Server to query.

    Returns:
        A list of supported Aggregate objects. Each object has 'id',
        'name', and 'desc' properties defined.
    """
    print(serverName)
    return [AggregateInfo()]


def getServers():
    # type: () -> List[String]
    """Returns a list of the OPC-HDA servers configured on the system.

    This call will return all configured and enabled servers, including
    those that are not currently connected.

    Returns:
        A list of the string names of servers.
    """
    return []


def insert(serverName, itemId, value, date, quality):
    # type: (String, String, Any, Any, int) -> QualityCode
    """Insert values on the OPC-HDA Server if the given item ID does not
    exist.

    Args:
        serverName: The name of the defined OPC-HDA Server.
        itemId: The item ID on which to perform the operation.
        value: The value to insert.
        date: The date to insert.
        quality: The quality to insert.

    Returns:
        The result of the insert.
    """
    print(serverName, itemId, value, date, quality)
    return QualityCode()


def insertReplace(serverName, itemId, value, date, quality):
    # type: (String, String, Any, Date, int) -> QualityCode
    """Will insert values on the OPC-HDA Server, or replace them if they
    already exist.

    Args:
        serverName: The name of the defined OPC-HDA Server.
        itemId: The item ID on which to perform the operation.
        value: The value to insert or replace.
        date: The date to insert or replace.
        quality: The quality to insert or replace.

    Returns:
        The items quality form the operation.
    """
    print(serverName, itemId, value, date, quality)
    return QualityCode()


def isServerAvailable(serverName):
    # type: (String) -> bool
    """Checks to see if the specified OPC-HDA Server is defined,
    enabled, and connected.

    Args:
        serverName: The name of the OPC-HDA Server to check.

    Returns:
        True if the Server is available and can be queried, False if
        not.
    """
    print(serverName)
    return True


def readAttributes(serverName, itemId, attributeIds, startDate, endDate):
    # type: (String, String, String, Date, Date) -> List[Any]
    """Reads the specified attributes for the given item over a time
    range.

    Attributes and their IDs are defined in the OPC-HDA specification,
    and can be discovered by calling system.opchda.getAttributes().

    Args:
        serverName: The name of the defined OPC-HDA Server to read.
        itemId: The itemID to retrieve attributes for.
        attributeIds: The integer IDs of the attributes to read. The
            attribute ids are defined in the OPC-HDA specification. The
            attributes can also be obtained by calling
            system.opchda.getAttributes(). Some servers may not support
            all attributes.
        startDate: The starting date/time of the query.
        endDate: The ending date/time of the query.

    Returns:
        A list of read results which is one-to-one with the requested
        attributes. The ReadResult object has a 'serviceResult' quality
        property that indicates whether the call was successful, and is
        itself a list of QualifiedValues.
    """
    print(serverName, itemId, attributeIds, startDate, endDate)
    return []


def readProcessed(
    serverName,  # type: String
    itemIds,  # type: List[String]
    startDate,  # type: Date
    endDate,  # type: Date
    resampleIntervalMS,  # type: int
    aggregates,  # type: List[Any]
):
    # type: (...) -> List[Any]
    """Reads processed values from the OPC-HDA Server.

    Processed values are calculated values, based on the aggregate
    function requested for each item. The list of aggregates can be
    obtained by calling system.opchda.getAggregates().

    Args:
        serverName: The name of the defined OPC-HDA Server to read.
        itemIds: A list of item ids to read.
        startDate: The starting date/time of the query.
        endDate: The ending date/time of the query.
        resampleIntervalMS: The interval, in milliseconds, that each
            value should cover.
        aggregates: A list which should be one-to-one with the item ids
            requested, specifying the integer id of the aggregation
            function to use. The aggregation ids are defined in the
            OPC-HDA specification. The list of aggregates can also be
            obtained by calling system.opchda.getAggregates().

    Returns:
        A list of read results which is one-to-one with the item IDs
        passed in. The ReadResult object has a 'serviceResult' quality
        property that indicates whether the call was successful, and is
        itself a list of QualifiedValues.
    """
    print(
        serverName,
        itemIds,
        startDate,
        endDate,
        resampleIntervalMS,
        aggregates,
    )
    return []


def readRaw(
    serverName,  # type: String
    itemIds,  # type: List[String]
    startDate,  # type: Date
    endDate,  # type: Date
    maxValues,  # type: int
    boundingValues,  # type: bool
):
    # type: (...) -> List[Any]
    """Reads raw values from the OPC-HDA Server.

    Args:
        serverName: The name of the defined OPC-HDA Server to read.
        itemIds: A list of item ids to read.
        startDate: The starting date/time of the query.
        endDate: The ending date/time of the query.
        maxValues: The maximum number of values to return. 0 or less
            means unlimited.
        boundingValues: A boolean indicating whether or not the
            "bounding values" should be included in the result set. The
            bounding values provide a value exactly at the start and end
            dates, but may be resource-intensive to retrieve.

    Returns:
        A list of read results which is one-to-one with the item IDs
        passed in. The ReadResult object has a 'serviceResult' quality
        property that indicates whether the call was successful, and is
        itself a list of QualifiedValues.
    """
    print(serverName, itemIds, startDate, endDate, maxValues, boundingValues)
    return []


def replace(serverName, itemId, value, date, quality):
    # type: (String, String, Any, Date, int) -> QualityCode
    """Replaces values on the OPC-HDA Server if the given item ID
    exists.

    Args:
        serverName: The name of the defined OPC-HDA Server.
        itemId: The item ID to perform the operation on.
        value: The value to replace.
        date: The date to replace.
        quality: The quality to replace.

    Returns:
        The items quality resulting from the operation.
    """
    print(serverName, itemId, value, date, quality)
    return QualityCode()



"""Provides classes that are fundamental to the design of the Java
programming language.
"""

from __future__ import print_function

__all__ = [
    "Appendable",
    "AutoCloseable",
    "CharSequence",
    "Class",
    "Enum",
    "Exception",
    "IllegalArgumentException",
    "NullPointerException",
    "Number",
    "Object",
    "Runnable",
    "RuntimeException",
    "StackTraceElement",
    "String",
    "StringBuffer",
    "StringBuilder",
    "Thread",
    "Throwable",
]

import __builtin__ as builtins
import time

from typing import Any, Optional, TypeVar, Union

String = Union[str, unicode]
T = TypeVar("T")
U = TypeVar("U")


class Object(object):
    """Class Object is the root of the class hierarchy. Every class has
    Object as a superclass. All objects, including arrays, implement the
    methods of this class.
    """

    def __cmp__(self, other):
        # type: (Object) -> bool
        pass

    def __ge__(self, other):
        # type: (Object) -> bool
        pass

    def __gt__(self, other):
        # type: (Object) -> bool
        pass

    def __lt__(self, other):
        # type: (Object) -> bool
        pass

    def equals(self, obj):
        # type: (Object) -> bool
        pass

    def getClass(self):
        # type: () -> Class
        pass

    def hashCode(self):
        # type: () -> int
        pass

    def notify(self):
        # type: () -> None
        pass

    def notifyAll(self):
        # type: () -> None
        pass

    def toString(self):
        # type: () -> String
        return repr(self)

    def wait(self, timeoutMillis=0, nanos=0):
        # type: (long, int) -> None
        pass


class Appendable(object):
    def append(self, c_csq, start=0, end=-1):
        # type: (Union[CharSequence, str], int, int) -> Appendable
        raise NotImplementedError


class AutoCloseable(object):
    def close(self):
        # type: () -> None
        raise NotImplementedError


class CharSequence(object):
    def charAt(self, index):
        # type: (int) -> str
        raise NotImplementedError

    def chars(self):
        pass

    def codePoints(self):
        pass

    @staticmethod
    def compare(cs1, cs2):
        # type: (CharSequence, CharSequence) -> int
        pass

    def length(self):
        # type: () -> int
        raise NotImplementedError

    def subSequence(self, start, end):
        # type: (int, int) -> CharSequence
        raise NotImplementedError

    def toString(self):
        # type: () -> String
        raise NotImplementedError


class Runnable(object):
    def run(self):
        # type: () -> None
        raise NotImplementedError


class Class(Object):
    def asSubClass(self, clazz):
        # type: (Class) -> U
        pass

    def cast(self, obj):
        # type: (Object) -> T
        pass


class Enum(Object):
    def compareTo(self, o):
        # type: (Enum) -> int
        pass

    def getDeclaringClass(self):
        # type: () -> Class
        pass

    def name(self):
        # type: () -> String
        pass

    def ordinal(self):
        # type: () -> int
        pass

    @staticmethod
    def valueOf(enumType, name):
        # type: (Class, String) -> Enum
        pass


class Number(Object):
    def byteValue(self):
        # type: () -> int
        pass

    def doubleValue(self):
        # type: () -> float
        raise NotImplementedError

    def floatValue(self):
        # type: () -> float
        raise NotImplementedError

    def intValue(self):
        # type: () -> int
        raise NotImplementedError

    def longValue(self):
        # type: () -> long
        raise NotImplementedError

    def shortValue(self):
        # type: () -> int
        pass


class StackTraceElement(Object):
    def __init__(self, *args):
        pass

    def getClassLoaderName(self):
        # type: () -> str
        pass

    def getClassName(self):
        # type: () -> str
        pass

    def getFileName(self):
        # type: () -> str
        pass

    def getLineNumber(self):
        # type: () -> int
        pass

    def getMethodName(self):
        # type: () -> str
        pass

    def getModuleName(self):
        # type: () -> str
        pass

    def getModuleVersion(self):
        # type: () -> str
        pass

    def isNativeMethod(self):
        # type: () -> bool
        pass


class StringBuffer(Object, CharSequence):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def append(self, *args):
        pass

    def appendCodePoint(self, codePoint):
        # type: (int) -> StringBuffer
        pass

    def capacity(self):
        # type: () -> int
        pass

    def charAt(self, index):
        # type: (int) -> str
        pass

    def codePointAt(self, index):
        # type: (int) -> int
        pass

    def codePointBefore(self, index):
        # type: (int) -> int
        pass

    def codePointCount(self, beginIndex, endIndex):
        # type: (int, int) -> int
        pass

    def compareTo(self, another):
        # type: (StringBuffer) -> int
        pass

    def delete(self, start, end):
        # type: (int, int) -> StringBuffer
        pass

    def deleteCharAt(self, index):
        # type: (int) -> StringBuffer
        pass

    def ensureCapacity(self, minimumCapacity):
        # type: (int) -> None
        pass

    def getChars(self, srcBegin, srcEnd, dst, dstBegin):
        # type: (int, int, str, int) -> None
        pass

    def indexOf(self, string, fromIndex=0):
        # type: (str, int) -> int
        pass

    def insert(self, *args):
        pass

    def lastIndexOf(self, string, fromIndex=0):
        # type: (str, int) -> int
        pass

    def length(self):
        # type: () -> int
        pass

    def offsetByCodePoints(self, index, codePointOffset):
        # type: (int, int) -> int
        pass

    def replace(self, start, end, string):
        # type: (int, int, str) -> StringBuffer
        pass

    def reverse(self):
        # type: () -> StringBuffer
        pass

    def setCharAt(self, index, ch):
        # type: (int, str) -> None
        pass

    def setLength(self, newLength):
        # type: (int) -> None
        pass

    def subSequence(self, start, end):
        # type: (int, int) -> CharSequence
        pass

    def substring(self, start, end=-1):
        # type: (int, int) -> str
        pass

    def trimToSize(self):
        # type: () -> None
        pass


class StringBuilder(Object, CharSequence):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def append(self, *args):
        # type: (Any) -> StringBuilder
        pass

    def capacity(self):
        # type: () -> int
        pass

    def charAt(self, index):
        # type: (int) -> str
        pass

    def codePointAt(self, index):
        # type: (int) -> int
        pass

    def codePointBefore(self, index):
        # type: (int) -> int
        pass

    def codePointCount(self, beginIndex, endIndex):
        # type: (int, int) -> int
        pass

    def compareTo(self, another):
        # type: (StringBuilder) -> int
        pass

    def delete(self, start, end):
        # type: (int, int) -> StringBuilder
        pass

    def deleteCharAt(self, index):
        # type: (int) -> StringBuilder
        pass

    def ensureCapacity(self, minimumCapacity):
        # type: (int) -> None
        pass

    def getChars(self, srcBegin, srcEnd, dst, dstBegin):
        # type: (int, int, str, int) -> None
        pass

    def indexOf(self, str_, fromIndex=0):
        # type: (String, int) -> int
        pass

    def insert(self, *args):
        # type: (Any) -> StringBuilder
        pass

    def lastIndexOf(self, str_, fromIndex=0):
        # type: (String, int) -> int
        pass

    def length(self):
        # type: () -> int
        pass

    def offsetByCodePoints(self, index, codePointOffset):
        # type: (int, int) -> int
        pass

    def replace(self, start, end, str_):
        # type: (int, int, int) -> StringBuilder
        pass

    def reverse(self):
        # type: () -> StringBuilder
        pass

    def setCharAt(self, index, ch):
        # type: (int, str) -> None
        pass

    def setLength(self, newLength):
        # type: (int) -> None
        pass

    def subSequence(self, start, end):
        # type: (int, int) -> CharSequence
        pass

    def substring(self, start, end=-1):
        # type: (int, int) -> String
        pass

    def trimToSize(self):
        # type: () -> None
        pass


class Throwable(Object, builtins.Exception):
    """The Throwable class is the superclass of all errors and
    exceptions in the Java language.
    """

    def __init__(self, message=None, cause=None):
        # type: (Optional[str], Optional[Throwable]) -> None
        self._cause = cause
        self._message = message
        builtins.Exception.__init__(self, message)

    @property
    def cause(self):
        # type: () -> Throwable
        return Throwable()

    def addSuppressed(self, exception):
        # type: (Throwable) -> None
        pass

    def fillInStackTrace(self):
        # type: () -> Throwable
        pass

    def getCause(self):
        # type: () -> Throwable
        return self.cause

    def getLocalizedMessage(self):
        # type: () -> str
        return self.message

    def getMessage(self):
        # type: () -> str
        pass

    def getStackTrace(self):
        pass

    def getSuppressed(self):
        pass

    def initCause(self, cause=None):
        # type: (Optional[Throwable]) -> None
        pass

    @property
    def message(self):
        # type: () -> str
        return "message"

    def printStackTrace(self, *args):
        pass

    def setStackTrace(self, stackTrace):
        pass

    def toString(self):
        # type: () -> String
        return "A short description of this throwable."


class Exception(Throwable):
    """The class Exception and its subclasses are a form of Throwable
    that indicates conditions that a reasonable application might want
    to catch.

    The class Exception and any subclasses that are not also subclasses
    of RuntimeException are checked exceptions. Checked exceptions need
    to be declared in a method or constructor's throws clause if they
    can be thrown by the execution of the method or constructor and
    propagate outside the method or constructor boundary.
    """

    def __init__(self, message=None, cause=None):
        # type: (Optional[str], Optional[Throwable]) -> None
        super(Exception, self).__init__(message, cause)


class RuntimeException(Exception):
    """RuntimeException is the superclass of those exceptions that can
    be thrown during the normal operation of the Java Virtual Machine.

    RuntimeException and its subclasses are unchecked exceptions.
    Unchecked exceptions do not need to be declared in a method or
    constructor's throws clause if they can be thrown by the execution
    of the method or constructor and propagate outside the method or
    constructor boundary.
    """

    def __init__(self, message=None, cause=None):
        # type: (Optional[str], Optional[Throwable]) -> None
        super(RuntimeException, self).__init__(message, cause)


class IllegalArgumentException(RuntimeException):
    """Thrown to indicate that a method has been passed an illegal or
    inappropriate argument.
    """

    def __init__(self, message=None, cause=None):
        # type: (Optional[str], Optional[Throwable]) -> None
        super(IllegalArgumentException, self).__init__(message, cause)


class NullPointerException(RuntimeException):
    """Thrown when an application attempts to use null in a case where
    an object is required.
    """

    def __init__(self, message=None, cause=None):
        # type: (Optional[str], Optional[Throwable]) -> None
        super(NullPointerException, self).__init__(message, cause)


class Thread(Object):
    """A thread is a thread of execution in a program. The Java Virtual
    Machine allows an application to have multiple threads of execution
    running concurrently.

    Every thread has a name for identification purposes. More than one
    thread may have the same name. If a name is not specified when a
    thread is created, a new name is generated for it.

    Unless otherwise noted, passing a null argument to a constructor or
    method in this class will cause a NullPointerException to be thrown.
    """

    @staticmethod
    def sleep(millis):
        # type: (long) -> None
        time.sleep(millis // 1000)



"""Dataset Functions.

The following functions give you access to view and interact with
datasets.
"""

from __future__ import print_function

__all__ = [
    "addColumn",
    "addRow",
    "addRows",
    "appendDataset",
    "clearDataset",
    "dataSetToHTML",
    "deleteRow",
    "deleteRows",
    "exportCSV",
    "exportExcel",
    "exportHTML",
    "filterColumns",
    "formatDates",
    "fromCSV",
    "getColumnHeaders",
    "setValue",
    "sort",
    "toCSV",
    "toDataSet",
    "toExcel",
    "toPyDataSet",
    "updateRow",
]

import os.path

from typing import Any, Dict, List, Optional, Type, Union

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.script.builtin import DatasetUtilities
from java.lang import String
from java.util import Date, Locale

ColType = Union[Date, float, int, str, unicode]
PyDataSet = DatasetUtilities.PyDataSet


def addColumn(
    dataset,  # type: BasicDataset
    colIndex,  # type: int
    col,  # type: List[Any]
    colName,  # type: String
    colType,  # type: Type[ColType]
):
    # type: (...) -> BasicDataset
    """Takes a dataset and returns a new dataset with a new column added
    or inserted into it.

    If the `colIndex` argument is omitted, the column will be appended
    to the end of the dataset.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Please be aware that this dataset
            will not actually be modified (datasets are immutable), but
            rather will be the starting point for creating a new
            dataset.
        colIndex: The index (starting at 0) at which to insert the new
            column. Will throw an IndexError if less than zero or
            greater than the length of the dataset. If omitted, the new
            column will be appended to the end. Optional.
        col: A Python sequence representing the data for the new column.
            Its length must equal the number of rows in the dataset.
        colName: The name of the column.
        colType: The type of the of the column. The type can be a Python
            builtin type, such as str, int, float, or a Java class, such
            as java.util.Date.

    Returns:
        A new dataset with the new column inserted or appended.
    """
    print(dataset, colIndex, col, colName, colType)
    return BasicDataset()


def addRow(dataset, rowIndex, row):
    # type: (BasicDataset, int, List[Any]) -> BasicDataset
    """Takes a dataset and returns a new dataset with a new row added or
    inserted into it.

    If the `rowIndex` argument is omitted, the column will be appended
    to the end of the dataset.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Please be aware that this dataset
            will not actually be modified (datasets are immutable), but
            rather will be the starting point for creating a new
            dataset.
        rowIndex: The index (starting at 0) at which to insert the new
            row. Will throw an IndexError if less than zero or greater
            than the length of the dataset. If omitted, the new row will
            be appended to the end. Optional.
        row: A Python sequence representing the data for the new row.
            Its length must equal the number of columns in the dataset.

    Returns:
        A new dataset with the new row inserted or appended.
    """
    print(dataset, rowIndex, row)
    return BasicDataset()


def addRows(dataset, rowIndex, rows):
    # type: (BasicDataset, int, List[List[Any]]) -> BasicDataset
    """Takes a dataset and returns a new dataset with a new row added or
    inserted into it.

    If the `rowIndex` argument is omitted, the column will be appended
    to the end of the dataset.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Please be aware that this dataset
            will not actually be modified (datasets are immutable), but
            rather will be the starting point for creating a new
            dataset.
        rowIndex: The index (starting at 0) at which to insert the new
            row. Will throw an IndexError if less than zero or greater
            than the length of the dataset. If omitted, the new row will
            be appended to the end. Optional.
        rows: A Python sequence of sequences representing the data for
            the new rows. The length of each sequence must equal the
            number of columns in the dataset.

    Returns:
        A new dataset with the new row inserted or appended.
    """
    print(dataset, rowIndex, rows)
    return BasicDataset()


def appendDataset(dataset1, dataset2):
    # type: (BasicDataset, BasicDataset) -> BasicDataset
    """Takes two different datasets and returns a new dataset with the
    second dataset appended to the first.

    Will throw an error if the number and types of columns do not match.

    Args:
        dataset1: The dataset that will come first in the returned
            dataset.
        dataset2: The second dataset that will be appended to the end in
            the returned dataset.

    Returns:
        A new dataset that is a combination of the original two
        datasets.
    """
    print(dataset1, dataset2)
    return BasicDataset()


def clearDataset(dataset):
    # type: (BasicDataset) -> BasicDataset
    """Takes a dataset and returns a new dataset with all of the same
    column names, but all of the rows deleted.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset.

    Returns:
        A new dataset with no data.
    """
    print(dataset)
    return BasicDataset()


def dataSetToHTML(showHeaders, dataset, title):
    # type: (bool, BasicDataset, String) -> String
    """Formats the contents of a dataset as an HTML page, returning the
    results as a string.

    Uses the <table> element to create a data table page.

    Args:
        showHeaders: If True, the HTML table will include a header
            row.
        dataset: The dataset to export.
        title: The title for the HTML page.

    Returns:
        The HTML page as a string.
    """
    print(showHeaders, dataset, title)
    return "<html><head>{}</head><body>data</body></html>".format(title)


def deleteRow(dataset, rowIndex):
    # type: (BasicDataset, int) -> BasicDataset
    """Takes a dataset and returns a new dataset with a row removed.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Please be aware that this dataset
            will not actually be modified (datasets are immutable), but
            rather will be the starting point for creating a new
            dataset.
        rowIndex: The index (starting at 0) of the row to delete.

    Returns:
        A new dataset with the specified row removed.

    Raises:
        IndexError: If rowIndex is less than zero or greater than the
            row count of the dataset -1.
    """
    print(dataset, rowIndex)
    if rowIndex < 0:
        raise IndexError("Error")

    return BasicDataset()


def deleteRows(dataset, rowIndices):
    # type: (BasicDataset, List[int]) -> BasicDataset
    """Takes a dataset and returns a new dataset with one or more rows
    removed.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Please be aware that this dataset
            will not actually be modified (datasets are immutable), but
            rather will be the starting point for creating a new
            dataset.
        rowIndices: The indices (starting at 0) of the rows to delete.

    Returns:
        A new dataset with the specified row removed.

    Raises:
        IndexError: If any element is less than zero or greater than the
            number of rows in the dataset - 1.
    """
    print(dataset, rowIndices)
    if -1 in rowIndices:
        raise IndexError("Error")

    return BasicDataset()


def exportCSV(filename, showHeaders, dataset):
    # type: (String, bool, BasicDataset) -> String
    """Exports the contents of a dataset as a CSV file, prompting the
    user to save the file to disk.

    To write silently to a file, you cannot use the `dataset.export*`
    functions. Instead, use the `toCSV()` function.

    Args:
        filename: A suggested filename to save as.
        showHeaders: If True, the CSV file will include a header
            row.
        dataset: The dataset to export.

    Returns:
        The path to the saved file, or None if the action was canceled
        by the user.
    """
    print(filename, showHeaders, dataset)
    return os.path.expanduser("~")


def exportExcel(filename, showHeaders, dataset, nullsEmpty=False):
    # type: (String, bool, List[BasicDataset], bool) -> String
    """Exports the contents of a dataset as an Excel spreadsheet,
    prompting the user to save the file to disk.

    Uses the same format as the dataSetToExcel function.

    To write silently to a file, you cannot use the `dataset.export*`
    functions. Instead, use the `toExcel()` function.

    Args:
        filename: A suggested filename to save as.
        showHeaders: If True, the spreadsheet will include a header
            row.
        dataset: A sequence of datasets, one for each sheet in the
            resulting workbook.
        nullsEmpty: If True, the spreadsheet will leave cells with NULL
            values empty, instead of allowing Excel to provide a default
            value like 0. Defaults to False. Optional.

    Returns:
        The path to the saved file, or None if the action was canceled
        by the user.
    """
    print(filename, showHeaders, dataset, nullsEmpty)
    return os.path.expanduser("~")


def exportHTML(filename, showHeaders, dataset, title):
    # type: (String, bool, BasicDataset, String) -> String
    """Exports the contents of a dataset to an HTML page.

    Prompts the user to save the file to disk.

    Args:
        filename: A suggested filename to save as.
        showHeaders: If True, the HTML table will include a header
            row.
        dataset: The dataset to export.
        title: The title for the HTML page.

    Returns:
        The path to the saved file, or None if the action was canceled
        by the user.
    """
    print(filename, showHeaders, dataset, title)
    return os.path.expanduser("~")


def filterColumns(
    dataset,  # type: BasicDataset
    columns,  # type: Union[List[String], List[int]]
):
    # type: (...) -> BasicDataset
    """Takes a dataset and returns a view of the dataset containing only
    the columns found within the given list of columns.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset.
        columns: A list of columns to keep in the returned dataset. The
            columns may be in integer index form (starting at 0), or the
            name of the columns as Strings.

    Returns:
        A new dataset containing the filtered columns.
    """
    print(dataset, columns)
    return BasicDataset()


def formatDates(dataset, dateFormat, locale=Locale.ENGLISH):
    # type: (BasicDataset, String, Optional[Locale]) -> BasicDataset
    """Returns a new dataset with Date columns as strings formatted
    according to the dateFormat specified.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset to format.
        dateFormat: A valid Java DateFormat string such as
            "yyyy-MM-dd HH:mm:ss". See system.date.format for more
            information on the valid characters.
        locale: The Locale to use for formatting. If no Locale is
            specified, the default Locale will be used. Optional.

    Returns:
        A new dataset, containing the formatted dates.
    """
    print(dataset, dateFormat, locale)
    return BasicDataset()


def fromCSV(csv):
    # type: (String) -> BasicDataset
    """Converts a dataset stored in a CSV formatted string to a dataset
    that can be immediately assignable to a dataset property in your
    project.

    Usually this is used in conjunction with `readFileAsString` when
    reading in a CSV file that was exported using `toCSV`. The CSV
    string must be formatted in a specific way.

    Args:
        csv: A string holding a CSV dataset.

    Returns:
        A new dataset.
    """
    print(csv)
    return BasicDataset()


def getColumnHeaders(dataset):
    # type: (BasicDataset) -> List[String]
    """Takes in a dataset and returns the headers as a python list.

    Args:
        dataset: The input dataset.

    Returns:
        A list of column header strings.
    """
    print(dataset)
    return ["column1", "column2"]


def setValue(dataset, rowIndex, columnName, value):
    # type: (BasicDataset, int, String, Any) -> BasicDataset
    """Takes a dataset and returns a new dataset with a one value
    altered.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Will not be modified (datasets
            are immutable), but acts as the basis for the returned
            dataset.
        rowIndex: The index of the row to set the value at (starting at
            0).
        columnName: The name of the column to set the value at. Case
            insensitive.
        value: The new value for the specified row/column.

    Returns:
         A new dataset, with the new value set at the given location.
    """
    print(dataset, rowIndex, columnName, value)
    return BasicDataset()


def sort(
    dataset,  # type: BasicDataset
    keyColumn,  # type: Union[String, int]
    ascending=True,  # type: bool
):
    # type: (...) -> BasicDataset
    """Takes a dataset and returns a sorted version of dataset.

    The sort order is determined by a single column. This works on
    numeric, as well as alphanumeric columns.

    When sorting alphanumerically, contiguous numbers are treated as a
    single number: you may recognize this as a "natural sort".

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The dataset to sort.
        keyColumn: The index or column name to sort on.
        ascending: True for ascending order, False for descending order.
            If omitted, ascending order will be used. Optional.

    Returns:
        A new sorted dataset.
    """
    print(dataset, keyColumn, ascending)
    return BasicDataset()


def toCSV(
    dataset,  # type: BasicDataset
    showHeaders=True,  # type: bool
    forExport=False,  # type: bool
    localized=False,  # type: bool
):
    # type: (...) -> String
    """Formats the contents of a dataset as CSV (comma separated
    values), returning the resulting CSV as a string.

    If the "forExport" flag is set, then the format will be appropriate
    for parsing using the `fromCSV` function.

    Args:
        dataset: The dataset to export to CSV.
        showHeaders: If set to True, a header row will be present in
            the CSV. Default is True. Optional.
        forExport: If set to True, extra header information will be
            present in the CSV data which is necessary for the CSV to be
            compatible with the fromCSV method. Overrides showHeaders.
            Default is False. Optional.
        localized: If set to True, the string representations of the
            values in the CSV data will be localized. Default is
            False. Optional.

    Returns:
        The CSV data as a string.
    """
    print(dataset, showHeaders, forExport, localized)
    return ""


def toDataSet(*args):
    # type: (*Any) -> BasicDataset
    """This function is used to convert PyDataSets to DataSets, and to
    create new datasets from raw Python lists.

    When creating a new dataset, headers should have unique names.

    1) system.dataset.toDataSet(dataset)
    2) system.dataset.toDataSet(headers, data)

    Args:
        args: A variable-length argument list.

    Returns:
        The newly created dataset.
    """
    print(args)
    return BasicDataset()


def toExcel(
    showHeaders,  # type: bool
    dataset,  # type: List[BasicDataset]
    nullsEmpty=False,  # type: bool
    sheetNames=None,  # type: Optional[List[String]]
):
    # type: (...) -> Any
    """Formats the contents of one or more datasets as an Excel
    spreadsheet, returning the results as a byte array.

    Each dataset specified will be added as a worksheet in the Excel
    workbook.

    Args:
        showHeaders: If True, the spreadsheet will include a header row.
            If False, the header row will be omitted.
        dataset: A sequence of one or more datasets, one for each sheet
            in the resulting workbook.
        nullsEmpty: If True, the spreadsheet will leave cells with
            NULL values empty, instead of allowing Excel to provide a
            default value like 0. Defaults to False. Optional.
        sheetNames: Expects a list of strings, where each string is a
            name for one of the datasets. When used, there must be an
            equal number of string names in sheetName as there are
            datasets in the dataset parameter. Names provided in this
            parameter may be sanitized into acceptable Excel sheet
            names. Optional.

    Returns:
        A byte array representing an Excel workbook.
    """
    print(showHeaders, dataset, nullsEmpty, sheetNames)


def toPyDataSet(dataset):
    # type: (Dataset) -> PyDataSet
    """This function converts from a normal DataSet to a PyDataSet,
    which is a wrapper class which makes working with datasets more
    Python-esque.

    Args:
        dataset: A Dataset object to convert into a PyDataSet.

    Returns:
        The newly created PyDataSet.
    """
    print(dataset)
    return PyDataSet()


def updateRow(dataset, rowIndex, changes):
    # type: (BasicDataset, int, Dict[String, Any]) -> BasicDataset
    """Takes a dataset and returns a new dataset with a one row altered.

    To alter the row, this function takes a Python dictionary to
    represent the changes to make to the specified row. The keys in the
    dictionary are used to find the columns to alter.

    Note:
        Datasets are immutable, which means they cannot be directly
        modified once created. Instead, this scripting function returns
        a new dataset with some modification applied, which must be
        assigned to a variable to be used.

    Args:
        dataset: The starting dataset. Will not be modified (datasets
            are immutable), but acts as the basis for the returned
            dataset.
        rowIndex: The index of the row to update (starting at 0).
        changes: A Dictionary of changes to make. They keys in the
            dictionary should match column names in the dataset, and
            their values will be used to update the row.

    Returns:
         A new dataset with the values at the specified row updated
         according to the values in the dictionary.
    """
    print(dataset, rowIndex, changes)
    return BasicDataset()



__all__ = ["Annotation"]

from java.lang import Object


class Annotation(Object):
    def delete(self):
        pass

    @staticmethod
    def fromJson(json):
        pass

    def getData(self):
        pass

    def getPath(self):
        pass

    def getRangeEnd(self):
        pass

    def getRangeStart(self):
        pass

    def getStorageId(self):
        pass

    def getType(self):
        pass

    def isDeleted(self):
        pass

    @staticmethod
    def newBuilder():
        pass

    @staticmethod
    def newDelete(storageId):
        pass

    def toJson(self):
        pass

    def updatePath(self, path):
        pass



"""Project Functions.

The following functions allow you to list projects on the Gateway
through scripting.
"""

from __future__ import print_function

__all__ = ["getProjectName", "getProjectNames"]

from typing import List

from java.lang import String


def getProjectName():
    # type: () -> String
    """Returns the name of the project where the function was called
    from.

    When called from the Gateway scope from a resource that originates
    from a singular project (reports, SFCs, etc.), will return that
    resources project.

    Resources that run in the Gateway scope, but are configured in a
    singular project (such as a report), will use that project's name.

    When called from a scope that does not have an associated project
    (a Tag Event Script), the function will return the name of the
    Gateway scripting project. If a Gateway scripting project has not
    been configured, then returns an empty string.

    Returns:
         The name of the currently running project.
    """
    return "MyProject"


def getProjectNames():
    # type: () -> List[String]
    """Returns an unsorted collection of strings, where each string
    represents the name of a project on the Gateway.

    If no projects exist, returns an empty list.

    This function only ever returns project names, ignoring project
    titles. The function also ignores the "enabled" property, including
    disabled projects in the results.

    Returns:
         A list containing string representations of project names on
         the Gateway.
    """
    return ["MyProject", "DisabledProject"]



from __future__ import print_function

__all__ = [
    "Icon",
    "JComponent",
    "JDesktopPane",
    "JFrame",
    "JInternalFrame",
    "JLabel",
    "JLayeredPane",
    "JOptionPane",
    "JPanel",
    "JPopupMenu",
    "JTextField",
]

from typing import Any, List, Optional

from java.awt import Container, Frame
from java.lang import String
from javax.swing.text import JTextComponent


class Icon(object):
    def getIconHeight(self):
        raise NotImplementedError

    def getIconWidth(self):
        raise NotImplementedError

    def paintIcon(self, c, g, x, y):
        raise NotImplementedError


class JComponent(Container):
    def __init__(self):
        # type: () -> None
        pass


class JFrame(Frame):
    def __init__(self, *args):
        # type: (Any) -> None
        super(JFrame, self).__init__(*args)


class JInternalFrame(JComponent):
    def __init__(
        self,
        title=None,  # type: Optional[String]
        resizable=None,  # type: Optional[bool]
        closable=None,  # type: Optional[bool]
        maximizable=None,  # type: Optional[bool]
        iconifiable=None,  # type: Optional[bool]
    ):
        # type: (...) -> None
        super(JInternalFrame, self).__init__()
        print(title, resizable, closable, maximizable, iconifiable)


class JLabel(JComponent):
    """A display area for a short text string or an image, or both."""

    def __init__(self, *args):
        # type: (Any) -> None
        super(JLabel, self).__init__()
        print(args)


class JLayeredPane(JComponent):
    DEFAULT_LAYER = 0
    DRAG_LAYER = 400
    FRAME_CONTENT_LAYER = -30000
    LAYER_PROPERTY = "layeredContainerLayer"
    MODAL_LAYER = 200
    PALETTE_LAYER = 100
    POPUP_LAYER = 300

    def __init__(self):
        # type: () -> None
        super(JLayeredPane, self).__init__()

    def getComponentCountInLayer(self, layer):
        pass

    def getLayer(self, c):
        pass

    def highestLayer(self):
        pass

    def lowestLayer(self):
        pass

    def setPosition(self, c, position):
        pass


class JDesktopPane(JLayeredPane):
    def __init__(self):
        # type: () -> None
        super(JDesktopPane, self).__init__()

    def getAllFrames(self):
        pass

    def getAllFramesInLayer(self):
        pass

    def getDesktopManager(self):
        pass

    def getDragMode(self):
        pass

    def getSelectedFrame(self):
        pass

    def getUI(self):
        pass

    def getUIClassID(self):
        pass

    def updateUI(self):
        pass


class JOptionPane(JComponent):
    """JOptionPane makes it easy to pop up a standard dialog box that
    prompts users for a value or informs them of something. For
    information about using JOptionPane, see How to Make Dialogs, a
    section in The Java Tutorial.
    """

    # messageType.
    PLAIN_MESSAGE = -1
    ERROR_MESSAGE = 0
    INFORMATION_MESSAGE = 1
    WARNING_MESSAGE = 2
    QUESTION_MESSAGE = 3

    # optionType.
    DEFAULT_OPTION = -1
    YES_NO_OPTION = 0
    YES_NO_CANCEL_OPTION = 1
    OK_CANCEL_OPTION = 2

    # When one of the showXxxDialog methods returns an integer, the
    # possible values are:
    CLOSED_OPTION = -1
    OK_OPTION = 0
    YES_OPTION = 0
    NO_OPTION = 1
    CANCEL_OPTION = 2

    @staticmethod
    def showConfirmDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        optionType=None,  # type: Optional[int]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
    ):
        # type: (...) -> int
        print(parentComponent, message, title, optionType, messageType, icon)
        return JOptionPane.YES_OPTION

    @staticmethod
    def showInputDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
        selectionValues=None,  # type: Optional[List[Any]]
        initialSelectionValue=None,  # type: Optional[Any]
    ):
        # type: (...) -> String
        print(
            parentComponent,
            message,
            title,
            messageType,
            icon,
            selectionValues,
            initialSelectionValue,
        )
        return "Input"

    @staticmethod
    def showMessageDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
    ):
        # type: (...) -> None
        print(parentComponent, message, title, messageType, icon)

    @staticmethod
    def showOptionDialog(
        parentComponent,  # type: Optional[Any]
        message,  # type: Any
        title=None,  # type: Optional[String]
        optionType=None,  # type: Optional[int]
        messageType=None,  # type: Optional[int]
        icon=None,  # type: Optional[Icon]
        options=None,  # type: Optional[List[Any]]
        initialValue=None,  # type: Optional[Any]
    ):
        # type: (...) -> int
        print(
            parentComponent,
            message,
            title,
            optionType,
            messageType,
            icon,
            options,
            initialValue,
        )
        return JOptionPane.YES_OPTION


class JPanel(JComponent):
    def __init__(self, *args):
        # type: (Any) -> None
        super(JPanel, self).__init__()
        print(args)


class JToolTip(JComponent):
    def getAccessibleContext(self):
        pass

    def getComponent(self):
        print(self)
        return JComponent()

    def getTipText(self):
        pass


class JPopupMenu(JComponent):
    """An implementation of a popup menu -- a small window that pops up
    and displays a series of choices. A JPopupMenu is used for the menu
    that appears when the user selects an item on the menu bar. It is
    also used for "pull-right" menu that appears when the selects a menu
    item that activates it. Finally, a JPopupMenu can also be used
    anywhere else you want a menu to appear. For example, when the user
    right-clicks in a specified area.
    """

    def __init__(self, label=None):
        # type: (Optional[String]) -> None
        super(JPopupMenu, self).__init__()
        print(label)

    def addAncestorListener(self, listener):
        pass

    def addNotify(self):
        pass

    def createToolTip(self):
        print(self)
        return JToolTip()

    @staticmethod
    def getDefaultLocale():
        pass

    @staticmethod
    def isLightweightComponent(c):
        pass

    @staticmethod
    def setDefaultLocale(l):
        pass


class JTextField(JTextComponent):
    """JTextField is a lightweight component that allows the editing of
    a single line of text.
    """

    def __init__(self, *args):
        # type: (Any) -> None
        pass



from typing import Any, Generic, Iterator, List, TypeVar, Union

from java.lang import Class, Enum, Object, String

T = TypeVar("T")


class Watchable(object):
    def register(self, *args):
        # type: (Any) -> WatchKey
        raise NotImplementedError


class LinkOption(Enum):
    @staticmethod
    def values():
        # type: () -> List[LinkOption]
        pass


class Path(Watchable):
    def compareTo(self, other):
        # type: (Path) -> int
        pass

    def endsWith(self, other):
        # type: (Union[Object, Path, String]) -> bool
        pass

    def equals(self, other):
        # type: (Object) -> bool
        pass

    def getFileName(self):
        # type: () -> Path
        pass

    def getName(self, index):
        # type: (int) -> Path
        pass

    def getNameCount(self):
        # type: () -> int
        pass

    def getParent(self):
        # type: () -> Path
        pass

    def getRoot(self):
        # type: () -> Path
        pass

    def hashCode(self):
        # type: () -> int
        pass

    def isAbsolute(self):
        # type: () -> bool
        pass

    def iterator(self):
        # type: () -> Iterator[Path]
        pass

    def normalize(self):
        # type: () -> Path
        pass

    @staticmethod
    def of(*args):
        # type: (Any) -> Path
        pass

    def register(self, *args):
        # type: (Any) -> WatchKey
        pass

    def relativize(self, other):
        # type: (Path) -> Path
        pass

    def resolve(self, other):
        # type: (Union[Path, String]) -> Path
        pass

    def resolveSibling(self, other):
        # type: (Path) -> Path
        pass

    def startsWith(self, other):
        # type: (Path) -> bool
        pass

    def subpath(self, beginIndex, endIndex):
        # type: (int, int) -> Path
        pass

    def toAbsolutePath(self):
        # type: () -> Path
        pass

    def toFile(self):
        # type: () -> Any
        pass

    def toRealPath(self, *args):
        # type: (LinkOption) -> Path
        pass

    def toString(self):
        # type: () -> String
        pass

    def toUri(self):
        # type: () -> Any
        pass


class WatchEvent(object):
    def context(self):
        # type: () -> T
        raise NotImplementedError

    def count(self):
        # type: () -> int
        raise NotImplementedError

    def kind(self):
        # type: () -> WatchEvent.Kind
        raise NotImplementedError

    class Kind(object):
        def name(self):
            # type: () -> String
            raise NotImplementedError

        def type(self):
            # type: () -> Class
            raise NotImplementedError

    class Modifier(object):
        def name(self):
            # type: () -> String
            raise NotImplementedError


class WatchKey(object):
    def cancel(self):
        # type: () -> None
        raise NotImplementedError

    def isValid(self):
        # type: () -> bool
        raise NotImplementedError

    def pollEvents(self):
        # type: () -> List[WatchEvent]
        raise NotImplementedError

    def reset(self):
        # type: () -> bool
        raise NotImplementedError

    def watchable(self):
        # type: () -> Watchable
        raise NotImplementedError



"""User Functions.

The following functions give you access to view and edit users in the
Gateway.
"""

from __future__ import print_function

__all__ = [
    "addCompositeSchedule",
    "addHoliday",
    "addRole",
    "addSchedule",
    "addUser",
    "createScheduleAdjustment",
    "editHoliday",
    "editRole",
    "editSchedule",
    "editUser",
    "getHoliday",
    "getHolidayNames",
    "getHolidays",
    "getNewUser",
    "getRoles",
    "getSchedule",
    "getScheduleNames",
    "getScheduledUsers",
    "getSchedules",
    "getUser",
    "getUserSources",
    "getUsers",
    "isUserScheduled",
    "removeHoliday",
    "removeRole",
    "removeSchedule",
    "removeUser",
]

from typing import List, Optional, Union

from com.inductiveautomation.ignition.common.messages import UIResponse
from com.inductiveautomation.ignition.common.user import PyUser, UserSourceMeta
from com.inductiveautomation.ignition.common.user.schedule import (
    AbstractScheduleModel,
    BasicScheduleModel,
    HolidayModel,
    ScheduleAdjustment,
)
from java.lang import String
from java.util import Date, Locale


def addCompositeSchedule(name, scheduleOne, scheduleTwo, description=None):
    # type: (String, String, String, Optional[String]) -> UIResponse
    """Allows two schedules to be combined into a composite schedule.

    Args:
        name: The name of the new composite schedule.
        scheduleOne: The first schedule to combine.
        scheduleTwo: The second schedule to combine.
        description: Description of the new combined schedule. Optional.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the add.
    """
    print(name, scheduleOne, scheduleTwo, description)
    return UIResponse(Locale.ENGLISH)


def addHoliday(holiday):
    # type: (HolidayModel) -> UIResponse
    """Allows a holiday to be added.

    Args:
        holiday: The holiday to add.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the add.
    """
    print(holiday)
    return UIResponse(Locale.ENGLISH)


def addRole(userSource, role):
    # type: (String, String) -> UIResponse
    """Allows a role to the specified user source.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource: The user source to add a role to. Blank will use the
            default user source.
        role: The role to add. Role must not be blank and must not
            already exist.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the add.
    """
    print(userSource, role)
    return UIResponse(Locale.ENGLISH)


def addSchedule(schedule):
    # type: (AbstractScheduleModel) -> UIResponse
    """Allows a schedule to be added.

    Args:
        schedule: The schedule to add. Can be a BasicScheduleModel or
            CompositeScheduleModel object (or any other class that
            extends AbstractScheduleModel).

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the add.
    """
    print(schedule)
    return UIResponse(Locale.ENGLISH)


def addUser(userSource, user):
    # type: (String, PyUser) -> UIResponse
    """Adds a new user to a user source.

    Used in combination with getNewUser to create new user.

    Args:
        userSource: The user source to add a user to. If set to an empty
            string, the function will attempt to use the project's
            default user source (if called from a project).
        user: The user to add, as a User object. Refer also to the
            PyUser class.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the add attempt.
    """
    print(userSource, user)
    return UIResponse(Locale.ENGLISH)


def createScheduleAdjustment(startDate, endDate, isAvailable, note):
    # type: (Date, Date, bool, String) -> ScheduleAdjustment
    """Creates a schedule adjustment.

    Args:
        startDate: The starting date of the schedule adjustment.
        endDate: The ending date of the schedule adjustment.
        isAvailable: True if the user is available during this schedule
            adjustment.
        note: A note about the schedule adjustment.

    Returns:
        A ScheduleAdjustment object that can be added to a user.
    """
    return ScheduleAdjustment(startDate, endDate, isAvailable, note)


def editHoliday(holidayName, holiday):
    # type: (String, HolidayModel) -> UIResponse
    """Allows a holiday to be edited.

    Args:
        holidayName: The name of the holiday to edit. Name is
            case-sensitive.
        holiday: The edited holiday, as a HolidayModel object.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the edit.
    """
    print(holidayName, holiday)
    return UIResponse(Locale.ENGLISH)


def editRole(userSource, oldName, newName):
    # type: (String, String, String) -> UIResponse
    """Renames a role in the specified user source.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource: The user source in which the role is found. Blank
            will use the default user source.
        oldName: The role to edit. Role must not be blank and must
            exist.
        newName: The new name for the role. Must not be blank.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the edit.
    """
    print(userSource, oldName, newName)
    return UIResponse(Locale.ENGLISH)


def editSchedule(scheduleName, schedule):
    # type: (String, AbstractScheduleModel) -> UIResponse
    """Allows a schedule to be edited.

    Args:
        scheduleName: The name of the schedule to edit. Name is
            case-sensitive.
        schedule: The schedule to add. Can be a BasicScheduleModel or
            CompositeScheduleModel object (or any other class that
            extends AbstractScheduleModel).

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the edit.
    """
    print(scheduleName, schedule)
    return UIResponse(Locale.ENGLISH)


def editUser(userSource, user):
    # type: (String, PyUser) -> UIResponse
    """Alters a specific user in a user source, replacing the previous
    data with the new data passed in.

    Args:
        userSource: The user source in which the user is found. Blank
            will use the default user source.
        user: The user to update.

    Returns:
        A UIResponse object with lists of warnings, errors, and
        information returned after the edit attempt.
    """
    print(userSource, user)
    return UIResponse(Locale.ENGLISH)


def getHoliday(holidayName):
    # type: (String) -> Optional[HolidayModel]
    """Returns a specific holiday.

    Args:
        holidayName: The name of the holiday to return. Case-sensitive.

    Returns:
        The holiday, as a HolidayModel object,, or None if not found.
    """
    print(holidayName)
    return HolidayModel(holidayName, Date(), True)


def getHolidayNames():
    # type: () -> List[String]
    """Returns a collection of strings of all holiday names.

    Returns:
        A list of all holiday names, or an empty list if no holidays are
        defined.
    """
    return ["Cinco de mayo", "Labor Day", "Groundhog Day"]


def getHolidays():
    # type: () -> List[HolidayModel]
    """Returns a sequence of all of the holidays available.

    Returns:
        A list of holidays, as HolidayModel objects.
    """
    return [HolidayModel("Cinco de mayo", Date(), True)]


def getNewUser(userSource, username):
    # type: (String, String) -> PyUser
    """Creates a new user object.

    The user will not be added to the user source until addUser is
    called.

    Args:
        userSource: The name of the user source in which to create a
            user.
        username: The username for the new user. Does not check if
            username already exists or is valid.

    Returns:
        The new user.
    """
    print(userSource, username)
    return PyUser()


def getRoles(userSource):
    # type: (String) -> List[String]
    """Returns a sequence of strings representing all of the roles
    configured in a specific user source.

    Args:
        userSource: The user source to fetch the roles for.

    Returns:
        A List of strings that holds all the roles in the user source.
    """
    print(userSource)
    return ["Administrator", "Designer", "Developer"]


def getSchedule(scheduleName):
    # type: (String) -> Optional[AbstractScheduleModel]
    """Returns a specific schedule.

    Args:
        scheduleName: The name of the schedule to return.
            Case-sensitive.

    Returns:
         The schedule, which can be a BasicSchedule, CompositeSchedule,
         or another type registered by a module. If a schedule was not
         found, the function will return None if called from a Vision
         Client or the Designer. if called in from a Perspective Session
         or anywhere else in the Gateway scope, will throw an
         IllegalArgumentException.
    """
    print(scheduleName)
    return BasicScheduleModel()


def getScheduleNames():
    # type: () -> List[String]
    """Returns a sequence of strings representing the names of all of
    the schedules available.

    Returns:
        A List of strings that holds the names of all the available
        schedules.
    """
    return ["A", "Always", "B", "C", "Example", "MyComposite", "MySchedule"]


def getScheduledUsers(userSource, date=Date()):
    # type: (String, Optional[Union[Date, int]]) -> List[PyUser]
    """Returns a list of users that are scheduled on.

    If no users are scheduled, it will return an empty list.

    Args:
        userSource: The name of the user source to check for scheduled
            users.
        date: The date to check schedules for. May be a Java Date or
            Unix Time in ms.. If omitted, the current date and time will
            be used. Optional.

    Returns:
        List of all Users scheduled for the given date, taking schedule
        adjustments into account.
    """
    print(userSource, date)
    return [PyUser()]


def getSchedules():
    # type: () -> List[AbstractScheduleModel]
    """Returns a sequence of all available schedule models, which can
    be used to return configuration information on the schedule, such as
    time for each day of the week.

    Returns:
        A list of schedules. Each schedule can be a BasicScheduleModel
        object, CompositeScheduleModel object, or another type
        registered by a module.
    """
    return [BasicScheduleModel()]


def getUser(userSource, username):
    # type: (String, String) -> PyUser
    """Looks up a specific user in a user source, by username.

    The full User object is returned except for the user's password.

    Args:
        userSource: The name of the user source to search for the user
            in. Can be a blank string to use the Vision Client's default
            user source.
        username: The username of the user to search for.

    Returns:
        The user.
    """
    print(userSource, username)
    return PyUser()


def getUserSources():
    # type: () -> List[UserSourceMeta]
    """Returns a sequence of objects representing all of the user source
    profiles configured in the Gateway. Each object has a "name"
    property, a "description" property, and a "type" property.

    Returns:
        A List of all user source profiles configured in the system in
        ascending order by their names.
    """
    return [
        UserSourceMeta(
            "default",
            "This user source profile was automatically created during a clean startup",
            "INTERNAL",
        )
    ]


def getUsers(userSource):
    # type: (String) -> List[PyUser]
    """Retrieves the list of users in a specific user source.

    The User objects that are returned contain all of the information
    about that user, except for the user's password.

    Args:
        userSource: The name of the user source to find the users in.

    Returns:
        A list of User objects.
    """
    print(userSource)
    return [PyUser()]


def isUserScheduled(user, date=None):
    # type: (PyUser, Optional[Union[Date, int]]) -> bool
    """Will check if a specified User is scheduled currently or on a
    specified date/time.

    Args:
        user: The User object to check for on the schedule.
        date: The date to check schedules for. May be a Java Date or
            Unix Time in ms. If omitted, the current date and time
            will be used. Optional.

    Returns:
        True if the user is scheduled for the specified date, False if
        not.
    """
    print(user, date)
    return True


def removeHoliday(holidayName):
    # type: (String) -> UIResponse
    """Allows a holiday to be deleted.

    Args:
        holidayName: The name of the holiday to delete. Case-sensitive.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the deletion.
    """
    print(holidayName)
    return UIResponse(Locale.ENGLISH)


def removeRole(userSource, role):
    # type: (String, String) -> UIResponse
    """Removes a role from the specified user source.

    When altering the Gateway System User Source, the Allow User Admin
    setting must be enabled.

    Args:
        userSource: The user source in which the role is found. Blank
            will use the default user source.
        role: The role to remove. The role must exist.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the deletion.
    """
    print(userSource, role)
    return UIResponse(Locale.ENGLISH)


def removeSchedule(scheduleName):
    # type: (String) -> UIResponse
    """Allows a schedule to be deleted.

    Note that schedules which are used in Composite Schedules can not be
    deleted until they are removed from the Composite Schedule.

    Args:
        scheduleName: The name of the schedule to delete. Name is
            case-sensitive.

    Returns:
        A UIResponse object with lists of warnings, errors, and info
        about the success or failure of the deletion.
    """
    print(scheduleName)
    return UIResponse(Locale.ENGLISH)


def removeUser(userSource, username):
    # type: (String, String) -> UIResponse
    """Removes a specific user from a user source based on username.

    Args:
        userSource: The user source in which the user is found. Blank
            will use the default user source.
        username: The username of the user to remove.

    Returns:
        A UIResponse object with lists of warnings, errors, and
        information returned after the removal attempt.
    """
    print(userSource, username)
    return UIResponse(Locale.ENGLISH)



__all__ = [
    "BasicOPCBrowseElement",
    "BasicServerNodeId",
    "BrowseElement",
    "BrowseElementType",
    "OPCBrowseElement",
    "ServerBrowseElement",
    "ServerNodeId",
]

from typing import Any, Dict, List, Optional

from java.lang import Class, Enum, Object, String, StringBuilder


class OPCBrowseElement(object):
    def getDataType(self):
        # type: () -> Class
        raise NotImplementedError

    def getDescription(self):
        # type: () -> String
        raise NotImplementedError

    def getDisplayName(self):
        # type: () -> String
        raise NotImplementedError

    def getElementType(self):
        # type: () -> BrowseElementType
        raise NotImplementedError

    def getNodeId(self):
        # type: () -> String
        raise NotImplementedError

    def getServerNodeId(self):
        # type: () -> ServerNodeId
        raise NotImplementedError


class ServerNodeId(object):
    def getNodeId(self):
        # type: () -> String
        raise NotImplementedError

    def getServerName(self):
        # type: () -> String
        raise NotImplementedError


class BasicOPCBrowseElement(Object, OPCBrowseElement):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def getDataType(self):
        # type: () -> Class
        pass

    def getDescription(self):
        # type: () -> String
        pass

    def getDisplayName(self):
        # type: () -> String
        pass

    def getElementType(self):
        # type: () -> BrowseElementType
        pass

    def getNodeId(self):
        # type: () -> String
        pass

    def getServerNodeId(self):
        # type: () -> ServerNodeId
        pass


class BasicServerNodeId(Object, ServerNodeId):
    nodeId = None  # type: String
    serverName = None  # type: String

    def __init__(self, serverName, nodeId):
        # type: (String, String) -> None
        self.serverName = serverName
        self.nodeId = nodeId

    def getNodeId(self):
        # type: () -> String
        return self.nodeId

    def getServerName(self):
        # type: () -> String
        return self.serverName


class BrowseElement(Object):
    PROP_PROVIDER = 500  # type: int
    PROP_DRIVER = 501  # type: int
    _server = None  # type: String
    _browsePath = None  # type: String
    _itemName = None  # type: String
    _itemId = None  # type: String
    _itemType = None  # type: Optional[int]
    _properties = None  # type: Dict[int, BrowseElement.PropertyElement]
    _complete = False  # type: bool

    def __init__(self, itemType=None):
        # type: (Optional[int]) -> None
        self._itemType = itemType
        self._properties = {0: BrowseElement.PropertyElement(0, "", "")}

    def addProperty(self, id_, name, value):
        # type: (int, String, String) -> None
        pass

    def asServerNodeId(self):
        # type: () -> ServerNodeId
        pass

    def createRequestElement(self, type_):
        # type: (int) -> BrowseElement
        pass

    def fillInDetails(self, elm):
        # type: (BrowseElement) -> None
        pass

    def getBrowsePath(self):
        # type: () -> String
        return self._browsePath

    def getItemId(self):
        # type: () -> String
        return self._itemId

    def getItemName(self):
        # type: () -> String
        return self._itemName

    def getItemType(self):
        # type: () -> Optional[int]
        return self._itemType

    def getProperties(self):
        # type: () -> List[BrowseElement.PropertyElement]
        return self._properties.values()

    def getProperty(self, id_):
        # type: (int) -> BrowseElement.PropertyElement
        return self._properties[id_]

    def getServer(self):
        # type: () -> String
        return self._server

    def isComplete(self):
        # type: () -> bool
        return self._complete

    def setBrowsePath(self, browsePath):
        # type: (String) -> None
        self._browsePath = browsePath

    def setComplete(self, value):
        # type: (bool) -> None
        self._complete = value

    def setItemId(self, itemId):
        # type: (String) -> None
        self._itemId = itemId

    def setItemName(self, itemName):
        # type: (String) -> None
        self._itemName = itemName

    def setItemType(self, itemType):
        # type: (int) -> None
        self._itemType = itemType

    def setServer(self, server):
        # type: (String) -> None
        self._server = server

    @staticmethod
    def toBrowseElement(opcElem):
        # type: (OPCBrowseElement) -> BrowseElement
        pass

    @staticmethod
    def toBrowseElements(opcElements):
        # type: (List[OPCBrowseElement]) -> List[BrowseElement]
        pass

    @staticmethod
    def toServerNodeId(elem):
        # type: (BrowseElement) -> ServerNodeId
        pass

    def toXML(self, out, type_=None, includeProperties=None):
        # type: (StringBuilder, Optional[int], Optional[bool]) -> None
        pass

    class PropertyElement(Object):
        id = None  # type: int
        name = None  # type: String
        value = None  # type: String

        def __init__(self, id_, name, value):
            # type: (int, String, String) -> None
            self.id = id_
            self.name = name
            self.value = value

        def getId(self):
            # type: () -> int
            return self.id

        def getName(self):
            # type: () -> String
            return self.name

        def getValue(self):
            # type: () -> String
            return self.value


class BrowseElementType(Enum):
    DATAVARIABLE = None  # type: BrowseElementType
    DEVICE = None  # type: BrowseElementType
    FOLDER = None  # type: BrowseElementType
    METHOD = None  # type: BrowseElementType
    OBJECT = None  # type: BrowseElementType
    PROPERTY = None  # type: BrowseElementType
    SERVER = None  # type: BrowseElementType
    VIEW = None  # type: BrowseElementType

    def isSubscribable(self):
        # type: () -> bool
        pass

    @staticmethod
    def values():
        # type: () -> List[BrowseElementType]
        pass


class ServerBrowseElement(Object, OPCBrowseElement):
    nodeId = None  # type: ServerNodeId

    def __init__(self, serverName):
        # type: (String) -> None
        self.nodeId = BasicServerNodeId(serverName, "")

    def getDataType(self):
        # type: () -> Class
        pass

    def getDescription(self):
        # type: () -> String
        return ""

    def getDisplayName(self):
        # type: () -> String
        pass

    def getElementType(self):
        # type: () -> BrowseElementType
        return BrowseElementType.SERVER

    def getNodeId(self):
        # type: () -> String
        return self.getServerNodeId().getNodeId()

    def getServerNodeId(self):
        # type: () -> ServerNodeId
        return self.nodeId



"""BACnet Functions.

The following functions are used with the BACnet driver and a BACnet/IP
device.
"""

from __future__ import print_function

__all__ = [
    "enumerated",
    "enums",
    "readRaw",
    "synchronizeTime",
    "synchronizeTimeUtc",
    "writeRaw",
    "writeWithPriority",
]

from typing import Any, List, Optional

from java.lang import String
from system.bacnet.enumerated import ObjectType, PropertyIdentifier


def readRaw(
    deviceName,  # type: String
    objectType,  # type: ObjectType
    objectId,  # type: int
    propertyId,  # type: PropertyIdentifier
    propertyArrayIndex=None,  # type: Optional[int]
):
    # type: (...) -> List[Any]
    """Read from any BACnet object not explicitly supported by the
    BACnet driver.

    Args:
        deviceName: The name of the configured BACnet/IP device instance
            to read from.
        objectType: The numeric id of the objectType of the object
            instance being read from.
        objectId: The object instance number to read.
        propertyId: The PropertyIdentifier of the object instance being
            read.
        propertyArrayIndex: The array index of the property to read
            from. This parameter is optional and should not be used when
            reading from the entire array or if the property is not an
            array.
    """
    print(deviceName, objectType, objectId, propertyId, propertyArrayIndex)
    return [0]


def synchronizeTime(deviceName):
    # type: (String) -> None
    """Notifies the remote device of the correct current time, which is
    the system time (factoring in timezone and DST) of the server
    Ignition is running on.

    Args:
        deviceName: The name of the configured BACnet/IP device instance
            to write from.
    """
    print(deviceName)


def synchronizeTimeUtc(deviceName):
    # type: (String) -> None
    """Notifies the remote device of the correct current time in UTC.

    Args:
        deviceName: The name of the configured BACnet/IP device instance
            to write from.
    """
    print(deviceName)


def writeRaw(
    deviceName,  # type: String
    objectType,  # type: ObjectType
    objectId,  # type: int
    propertyId,  # type: PropertyIdentifier
    value,  # type: Any
    priority,  # type: int
    propertyArrayIndex,  # type: int
):
    # type: (...) -> None
    """Write to any BACnet object not explicitly supported by the BACnet
    driver.

    Args:
        deviceName: The name of the configured BACnet/IP device instance
            to write from.
        objectType: The numeric id of the objectType of the object
            instance being written to.
        objectId: The object instance number to write to.
        propertyId: The PropertyIdentifier of the object instance being
            written to.
        value: The value to write. Clearing a value can be accomplished
            by writing a None value.
        priority: The priority level to use when writing to commandable
            properties. Must match a level in the standard BACnet
            priority array (a value from 1 to 16). See the Priority
            Reference table below. This parameter is optional and
            defaults to 8 if not specified.
        propertyArrayIndex: The array index of the property to write to.
            This parameter is optional and should not be used when
            writing to the entire array or if the property is not an
            array.
    """
    print(
        deviceName,
        objectType,
        objectId,
        propertyId,
        value,
        priority,
        propertyArrayIndex,
    )


def writeWithPriority(deviceName, objectType, objectId, value, priority):
    # type: (String, int, int, Any, int) -> None
    """Write to the Present_Value attribute of an object with a custom
    priority level.

    Args:
        deviceName: The name of the configured BACnet/IP device instance
            to write from.
        objectType: The numeric id of the objectType of the object
            instance being written to.
        objectId: The object instance number to write to.
        value: The value to write. Clearing a value can be accomplished
            by writing a None value.
        priority: The priority level to write the value at. Must be in
            the range [1...16].
    """
    print(deviceName, objectType, objectId, value, priority)



__all__ = ["BasicQualifiedValue", "QualifiedValue", "Quality", "QualityCode"]

from enum import Enum
from typing import Any, Union

from dev.thecesrom.utils.decorators import classproperty
from java.lang import Object, String
from java.util import Date


class QualifiedValue(object):
    """Represents a value with a DataQuality & timestamp attached to
    it.
    """

    def derive(self, arg=None):
        # type: (Union[None, QualityCode, String]) -> QualifiedValue
        raise NotImplementedError

    def getQuality(self):
        # type: () -> QualityCode
        raise NotImplementedError

    def getTimestamp(self):
        # type: () -> Date
        raise NotImplementedError

    def getValue(self):
        # type: () -> Object
        raise NotImplementedError


class Quality(object):
    """Needed for gateway network interop with v7, but otherwise not
    used."""

    def getDescription(self):
        pass

    def getLevel(self):
        pass

    def getName(self):
        pass

    def getQualityCode(self):
        pass

    def isGood(self):
        pass

    class Level(Enum):
        Bad = 512
        Good = 192
        Unknown = 256

        def values(self):
            pass


class QualityCode(Object):
    """QualityCode contains a 32-bit integer code and optionally a
    diagnostic string.
    """

    Bad = None  # type: QualityCode
    Bad_AccessDenied = None  # type: QualityCode
    Bad_AggregateNotFound = None  # type: QualityCode
    Bad_DatabaseNotConnected = None  # type: QualityCode
    Bad_Disabled = None  # type: QualityCode
    Bad_Failure = None  # type: QualityCode
    Bad_GatewayCommOff = None  # type: QualityCode
    Bad_LicenseExceeded = None  # type: QualityCode
    Bad_NotConnected = None  # type: QualityCode
    Bad_NotFound = None  # type: QualityCode
    Bad_OutOfRange = None  # type: QualityCode
    Bad_ReadOnly = None  # type: QualityCode
    Bad_Stale = None  # type: QualityCode
    Bad_ReferenceNotFound = None  # type: QualityCode
    Bad_TrialExpired = None  # type: QualityCode
    Bad_Unauthorized = None  # type: QualityCode
    Bad_Unsupported = None  # type: QualityCode
    Error = None  # type: QualityCode
    Error_Configuration = None  # type: QualityCode
    Error_CycleDetected = None  # type: QualityCode
    Error_DatabaseQuery = None  # type: QualityCode
    Error_Exception = None  # type: QualityCode
    Error_ExpressionEval = None  # type: QualityCode
    Error_Formatting = None  # type: QualityCode
    Error_InvalidPathSyntax = None  # type: QualityCode
    Error_IO = None  # type: QualityCode
    Error_ScriptEval = None  # type: QualityCode
    Error_TagExecution = None  # type: QualityCode
    Error_TimeoutExpired = None  # type: QualityCode
    Error_TypeConversion = None  # type: QualityCode
    Good = None  # type: QualityCode
    Good_Backfill = None  # type: QualityCode
    Good_Initial = None  # type: QualityCode
    Good_Overload = None  # type: QualityCode
    Good_Provisional = None  # type: QualityCode
    Good_Unspecified = None  # type: QualityCode
    Good_WritePending = None  # type: QualityCode
    Uncertain = None  # type: QualityCode
    Uncertain_LastKnownValue = None  # type: QualityCode
    Uncertain_InitialValue = None  # type: QualityCode
    Uncertain_DataSubNormal = None  # type: QualityCode
    Uncertain_EngineeringUnitsExceeded = None  # type: QualityCode
    Uncertain_IncompleteOperation = None  # type: QualityCode

    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def derive(self, diagnosticMessage):
        pass

    def getCode(self):
        pass

    def getDiagnosticMessage(self):
        pass

    def getLevel(self):
        pass

    def getName(self):
        pass

    def isBad(self):
        # type: () -> bool
        pass

    def isBadOrError(self):
        # type: () -> bool
        pass

    def isError(self):
        # type: () -> bool
        pass

    def isGood(self):
        # type: () -> bool
        pass

    def isNotGood(self):
        # type: () -> bool
        pass

    def isUncertain(self):
        # type: () -> bool
        pass

    class Level(Object):
        def __init__(self):
            pass

        def code(self, userCode):
            pass

        @staticmethod
        def valueOf(name):
            pass

        @staticmethod
        def values():
            pass


class BasicQualifiedValue(QualifiedValue, Object):
    """The basic implementation of QualifiedValue."""

    quality = None  # type: QualityCode
    timestamp = None  # type: Date
    value = None  # type: Object

    def __init__(self, *args):
        # type: (Any) -> None
        print(args)
        self.quality = QualityCode.Bad_Stale
        self.timestamp = Date()
        self.value = Object()

    @classproperty
    def BAD(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad)

    @classproperty
    def CONFIG_ERROR(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Error_Configuration)

    @classproperty
    def DISABLED(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad_Disabled)

    @classproperty
    def EXPRESSION_EVAL_ERROR(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Error_ExpressionEval)

    @classproperty
    def INITIAL_VALUE(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Uncertain_InitialValue)

    @classproperty
    def NOT_CONNECTED(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad_NotConnected)

    @classproperty
    def NOT_FOUND(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad_NotFound)

    @classproperty
    def REFERENCE_NOT_FOUND(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad_ReferenceNotFound)

    @classproperty
    def STALE(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad_Stale)

    @classproperty
    def TRANSFORM_ERROR(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad)

    @classproperty
    def TYPE_CONVERSION(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Error_TypeConversion)

    @classproperty
    def UNKNOWN(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Uncertain)

    @classproperty
    def UNSUPPORTED(self):
        # type: () -> BasicQualifiedValue
        return BasicQualifiedValue(None, QualityCode.Bad_Unsupported)

    def clone(self):
        # type: () -> BasicQualifiedValue
        pass

    def derive(self, arg=None):
        # type: (Union[None, QualityCode, String]) -> QualifiedValue
        pass

    @staticmethod
    def fromObject(obj):
        # type: (Object) -> QualifiedValue
        pass

    def getQuality(self):
        # type: () -> QualityCode
        return self.quality

    def getTimestamp(self):
        # type: () -> Date
        return self.timestamp

    def getValue(self):
        # type: () -> Object
        return self.value

    def setQuality(self, code):
        # type: (QualityCode) -> None
        self.quality = code

    def setTimestamp(self, timestamp):
        # type: (Date) -> None
        self.timestamp = timestamp

    def setValue(self, value):
        # type: (Object) -> None
        self.value = value

    @staticmethod
    def unwrap(obj):
        # type: (Object) -> Object
        pass

    @staticmethod
    def updateTimestamp(copy):
        # type: (QualifiedValue) -> QualifiedValue
        pass

    @staticmethod
    def valueOf(obj):
        # type: (Object) -> Object
        pass



__all__ = [
    "Consumer",
    "Function",
    "Predicate",
    "ToDoubleFunction",
    "ToIntFunction",
    "ToLongFunction",
]

from typing import TypeVar

from java.lang import Object

R = TypeVar("R")
T = TypeVar("T")


class Consumer(object):
    def accept(self, t):
        # type: (T) -> None
        raise NotImplementedError

    def andThen(self, after):
        # type: (Consumer) -> Consumer
        raise NotImplementedError


class Function(object):
    def andThen(self, after):
        # type: (Function) -> Function
        pass

    def apply(self, t):
        # type: (T) -> R
        raise NotImplementedError

    def compose(self, before):
        # type: (Function) -> Function
        pass

    @staticmethod
    def identity():
        # type: () -> Function
        pass


class Predicate(object):
    @staticmethod
    def isEqual(targetRef):
        # type: (Object) -> Predicate
        pass

    def negate(self):
        # type: () -> Predicate
        pass

    def test(self, t):
        # type: (T) -> bool
        raise NotImplementedError


class ToDoubleFunction(object):
    def applyAsDouble(self, value):
        # type: (T) -> object
        raise NotImplementedError


class ToIntFunction(object):
    def applyAsInt(self, value):
        # type: (T) -> object
        raise NotImplementedError


class ToLongFunction(object):
    def applyAsLong(self, value):
        # type: (T) -> object
        raise NotImplementedError



"""Transaction Group Functions.

The following functions give you access to import and remove Transaction
Groups.
"""

from __future__ import print_function

__all__ = ["loadFromFile", "removeGroups"]

from typing import List

from java.lang import String


def loadFromFile(filePath, projectName, mode):
    # type: (String, String, int) -> None
    """Loads a transaction group configuration from an xml export, into
    the specified project (creating the project if necessary).

    The mode parameter dictates how overwrites occur.

    Note:
        This function is scoped for Perspective Sessions, but since all
        scripts in Perspective run on the Gateway, the file must be
        located on the Gateway's file system.

    Args:
        filePath: The path to a valid transaction group xml or csv file.
        projectName: The name of the project to load into.
        mode: How duplicates will be handled. 0 = Overwrite, 1 = Ignore,
            2 = Replace the existing project with this one.
    """
    print(filePath, projectName, mode)


def removeGroups(projectName, paths):
    # type: (String, List[String]) -> None
    """Removes the specified groups from the project.

    The group paths are "Folder/Path/To/GroupName", separated by forward
    slashes.

    Args:
        projectName: The project to remove from. If the project does not
            exist, throws an IllegalArgumentException.
        paths: A collection of paths to remove. The group paths are
            "Folder/Path/To/GroupName", separated by forward slashes.
    """
    print(projectName, paths)



from typing import Any

from java.lang import Object


class DocumentElement(Object):
    def __init__(self, *args):
        # type: (Any) -> None
        pass


class Document(DocumentElement):
    def __init__(self, *args):
        # type: (Any) -> None
        super(Document, self).__init__(*args)



from typing import Any, List, Optional, Union

from java.lang import CharSequence, Object, String, StringBuffer, StringBuilder
from java.util.function import Function, Predicate


class MatchResult(object):
    def end(self, group=None):
        # type: (Optional[int]) -> int
        raise NotImplementedError

    def group(self, group=None):
        # type: (Optional[int]) -> String
        raise NotImplementedError

    def groupCount(self):
        # type: () -> int
        raise NotImplementedError

    def start(self, group=None):
        # type: (Optional[int]) -> int
        raise NotImplementedError


class Matcher(Object, MatchResult):
    def appendReplacement(self, sb, replacement):
        # type: (Union[StringBuffer, StringBuilder], String) -> Matcher
        pass

    def appendTail(
        self,
        sb,  # type: Union[StringBuffer, StringBuilder]
    ):
        # type: (...) -> Union[StringBuffer, StringBuilder]
        pass

    def end(self, group=None):
        # type: (Optional[int]) -> int
        pass

    def find(self, start=None):
        # type: (Optional[int]) -> bool
        pass

    def group(self, group=None):
        # type: (Optional[Union[int, String]]) -> String
        pass

    def groupCount(self):
        # type: () -> int
        pass

    def hasAnchoringBounds(self):
        # type: () -> bool
        pass

    def hasTransparentBounds(self):
        # type: () -> bool
        pass

    def hitEnd(self):
        # type: () -> bool
        pass

    def lookingAt(self):
        # type: () -> bool
        pass

    def matches(self):
        # type: () -> bool
        pass

    def pattern(self):
        # type: () -> Pattern
        pass

    @staticmethod
    def quoteReplacement(s):
        # type: (String) -> String
        pass

    def region(self, start, end):
        # type: (int, int) -> Matcher
        pass

    def regionEnd(self):
        # type: () -> int
        pass

    def regionStart(self):
        # type: () -> int
        pass

    def replaceAll(self, arg):
        # type: (Union[Function, String]) -> String
        pass

    def replaceFirst(self, arg):
        # type: (Union[Function, String]) -> String
        pass

    def requireEnd(self):
        # type: () -> bool
        pass

    def reset(self, input=None):
        # type: (Optional[CharSequence]) -> Matcher
        pass

    def results(self):
        # type: () -> Any
        pass

    def start(self, group=None):
        # type: (Optional[int]) -> int
        pass

    def toMatchResult(self):
        # type: () -> MatchResult
        pass

    def useAnchoringBounds(self, b):
        # type: (bool) -> Matcher
        pass

    def usePattern(self, newPattern):
        # type: (Pattern) -> Matcher
        pass

    def useTransparentBounds(self, b):
        # type: (bool) -> Matcher
        pass


class Pattern(Object):
    CANON_EQ = None  # type: int
    CASE_INSENSITIVE = None  # type: int
    COMMENTS = None  # type: int
    DOTALL = None  # type: int
    LITERAL = None  # type: int
    MULTILINE = None  # type: int
    UNICODE_CASE = None  # type: int
    UNICODE_CHARACTER_CLASS = None  # type: int
    UNIX_LINES = None  # type: int

    def asMatchPredicate(self):
        # type: () -> Predicate
        pass

    def asPredicate(self):
        # type: () -> Predicate
        pass

    @staticmethod
    def compile(regex, flags=None):
        # type: (String, Optional[int]) -> Pattern
        pass

    def flags(self):
        # type: () -> int
        pass

    def matcher(self, input):
        # type: (CharSequence) -> Matcher
        pass

    @staticmethod
    def matches(regex, input):
        # type: (String, CharSequence) -> bool
        pass

    def pattern(self):
        # type: () -> String
        pass

    @staticmethod
    def quote(s):
        # type: (String) -> String
        pass

    def split(self, input, limit=None):
        # type: (CharSequence, Optional[int]) -> List[String]
        pass

    def splitAsStream(self, input):
        # type: (CharSequence) -> Any
        pass



"""Provides the classes for implementing networking applications."""

from __future__ import print_function

__all__ = [
    "InetAddress",
    "InetSocketAddress",
    "Socket",
    "SocketAddress",
    "SocketChannel",
    "SocketImpl",
    "SocketImplFactory",
    "SocketOption",
]

from typing import Any, List, Optional, Set, TypeVar

from java.io import Closeable, InputStream, OutputStream
from java.lang import Class, Enum, Object, String
from java.nio.channels import SocketChannel

T = TypeVar("T")


class SocketAddress(Object):
    def __init__(self):
        # type: () -> None
        pass


class FileNameMap(object):
    def getContentTypeFor(self, fileName):
        # type: (String) -> String
        raise NotImplementedError


class SocketImplFactory(object):
    def createSocketImpl(self):
        # type: () -> SocketImpl
        raise NotImplementedError


class SocketOption(object):
    def name(self):
        # type: () -> String
        raise NotImplementedError

    def type(self):
        # type: () -> Class
        raise NotImplementedError


class InetAddress(Object):
    def getAddress(self):
        # type: () -> bytearray
        pass

    @staticmethod
    def getAllByName(host):
        # type: (String) -> List[InetAddress]
        pass

    @staticmethod
    def getByAddress(*args):
        # type: (Any) -> InetAddress
        pass

    @staticmethod
    def getByName(host):
        # type: (String) -> InetAddress
        pass

    def getCanonicalHostName(self):
        # type: () -> String
        pass

    def getHostAddress(self):
        # type: () -> String
        pass

    def getHostName(self):
        # type: () -> String
        pass

    @staticmethod
    def getLocalHost():
        # type: () -> InetAddress
        pass

    @staticmethod
    def getLoopbackAddress():
        # type: () -> InetAddress
        pass

    def isAnyLocalAddress(self):
        # type: () -> bool
        pass

    def isLinkLocalAddress(self):
        # type: () -> bool
        pass

    def isLoopbackAddress(self):
        # type: () -> bool
        pass

    def isMCGlobal(self):
        # type: () -> bool
        pass

    def isMCLinkLocal(self):
        # type: () -> bool
        pass

    def isMCNodeLocal(self):
        # type: () -> bool
        pass

    def isMCOrgLocal(self):
        # type: () -> bool
        pass

    def isMCSiteLocal(self):
        # type: () -> bool
        pass

    def isMulticastAddress(self):
        # type: () -> bool
        pass

    def isReachable(self, *args):
        # type: (Any) -> bool
        pass

    def isSiteLocalAddress(self):
        # type: () -> bool
        pass


class InetSocketAddress(SocketAddress):
    def __init__(self, *args):
        # type: (Any) -> None
        print(args)
        super(InetSocketAddress, self).__init__()

    @staticmethod
    def createUnresolved(host, port):
        # type: (String, int) -> InetSocketAddress
        return InetSocketAddress(host, port)

    def getAddress(self):
        # type: () -> InetAddress
        pass

    def getHostname(self):
        # type: () -> String
        pass

    def getHostString(self):
        # type: () -> String
        pass

    def getPort(self):
        # type: () -> int
        pass

    def isUnresolved(self):
        # type: () -> bool
        pass


class Proxy(Object):
    NO_PROXY = None  # type: Proxy

    def __init__(self, type_, sa):
        # type: (Proxy.Type, SocketAddress) -> None
        print(type_, sa)

    def address(self):
        # type: () -> SocketAddress
        pass

    def type(self):
        # type: () -> Proxy.Type
        pass

    class Type(Enum):
        DIRECT = None  # type: Proxy.Type
        HTTP = None  # type: Proxy.Type
        SOCKS = None  # type: Proxy.Type

        @staticmethod
        def values():
            # type: () -> List[Proxy.Type]
            pass


class Socket(Object, Closeable):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def bind(self, bindpoint):
        # type: (SocketAddress) -> None
        pass

    def close(self):
        # type: () -> None
        pass

    def connect(self, endpoint, timeout=None):
        # type: (SocketAddress, Optional[int]) -> None
        pass

    def getChannel(self):
        # type: () -> SocketChannel
        pass

    def getInetAddress(self):
        # type: () -> InetAddress
        pass

    def getInputStream(self):
        # type: () -> InputStream
        pass

    def getKeepAlive(self):
        # type: () -> bool
        pass

    def getLocalAddress(self):
        # type: () -> InetAddress
        pass

    def getLocalPort(self):
        # type: () -> int
        pass

    def getLocalSocketAddress(self):
        # type: () -> SocketAddress
        pass

    def getOOBInline(self):
        # type: () -> bool
        pass

    def getOption(self, name):
        # type: (SocketOption) -> T
        pass

    def getOutputStream(self):
        # type: () -> OutputStream
        pass

    def getPort(self):
        # type: () -> int
        pass

    def getReceiveBufferSize(self):
        # type: () -> int
        pass

    def getRemoteSocketAddress(self):
        # type: () -> SocketAddress
        pass

    def getReuseAddress(self):
        # type: () -> bool
        pass

    def getSendBufferSize(self):
        # type: () -> int
        pass

    def getSoLinger(self):
        # type: () -> int
        pass

    def getSoTimeout(self):
        # type: () -> int
        pass

    def getTcpNoDelay(self):
        # type: () -> bool
        pass

    def getTrafficClass(self):
        # type: () -> int
        pass

    def isBound(self):
        # type: () -> bool
        pass

    def isClosed(self):
        # type: () -> bool
        pass

    def isConnected(self):
        # type: () -> bool
        pass

    def isInputShutdown(self):
        # type: () -> bool
        pass

    def isOutputShutdown(self):
        # type: () -> bool
        pass

    def sendUrgentData(self, data):
        # type: (int) -> None
        pass

    def setKeepAlive(self, on):
        # type: (bool) -> None
        pass

    def setOOBInline(self, on):
        # type: (bool) -> None
        pass

    def setOption(self, name, value):
        # type: (SocketOption, T) -> Socket
        pass

    def setPerformancePreferences(self, connectionTime, latency, bandwidth):
        # type: (int, int, int) -> None
        pass

    def setReceiveBufferSize(self, size):
        # type: (int) -> None
        pass

    def setReuseAddress(self, on):
        # type: (bool) -> None
        pass

    def setSendBufferSize(self, size):
        # type: (int) -> None
        pass

    @staticmethod
    def setSocketImplFactory(fac):
        # type: (SocketImplFactory) -> None
        pass

    def setSoLinger(self, on, linger):
        # type: (bool, int) -> None
        pass

    def setSoTimeout(self, timeout):
        # type: (int) -> None
        pass

    def setTcpNoDelay(self, on):
        # type: (bool) -> None
        pass

    def setTrafficClass(self, tc):
        # type: (int) -> None
        pass

    def shutdownInput(self):
        # type: () -> None
        pass

    def shutdownOutput(self):
        # type: () -> None
        pass

    def supportedOptions(self):
        # type: () -> Set[SocketOption]
        pass


class SocketImpl(Object):
    def __init__(self):
        # type: () -> None
        pass



__all__ = ["LoggerEx"]

from typing import Any, Optional, Union

from java.io import Closeable
from java.lang import AutoCloseable, Class, Object, String, Throwable
from org.apache.commons.lang3.builder import ToStringStyle
from org.slf4j import Logger


class LoggerEx(Object):
    """This class is a wrapper around a logger which provides additional
    useful tools.

    To create one, use the newBuilder() function and configure the
    builder.
    """

    DEFAULT_TO_STRING_STYLE = None  # type: ToStringStyle

    def createSubLogger(self, arg):
        # type: (Union[Class, String]) -> LoggerEx
        pass

    def debug(self, message, t=None):
        # type: (String, Optional[Throwable]) -> None
        pass

    def debugDuration(self, message):
        # type: (String) -> Closeable
        pass

    def debugEvent(self, message, *args):
        # type: (String, Any) -> None
        pass

    def debugf(self, message, *args):
        # type: (String, Any) -> None
        pass

    def error(self, message, t=None):
        # type: (String, Optional[Throwable]) -> None
        pass

    def errorEvent(self, message, *args):
        # type: (String, Any) -> None
        pass

    def errorf(self, message, *args):
        # type: (String, Any) -> None
        pass

    def fatal(self, message, t=None):
        # type: (String, Optional[Throwable]) -> None
        pass

    def getIdentObject(self):
        # type: () -> Object
        pass

    def getLoggerSLF4J(self):
        # type: () -> Logger
        pass

    def getName(self):
        # type: () -> String
        pass

    def getToStringStyle(self):
        # type: () -> ToStringStyle
        pass

    def info(self, message, t=None):
        # type: (String, Optional[Throwable]) -> None
        pass

    def infoDuration(self, message):
        # type: (String) -> Closeable
        pass

    def infoEvent(self, message, *args):
        # type: (String, Any) -> None
        pass

    def infof(self, message, *args):
        # type: (String, Any) -> None
        pass

    def isDebugEnabled(self):
        # type: () -> bool
        pass

    def isIdentObjectEnabled(self):
        # type: () -> bool
        pass

    def isInfoEnabled(self):
        # type: () -> bool
        pass

    def isTraceEnabled(self):
        # type: () -> bool
        pass

    def mdcClose(self):
        # type: () -> None
        pass

    def mdcPut(self, key, value):
        # type: (String, String) -> None
        pass

    def mdcPutCloseable(self, key, value):
        # type: (String, String) -> LoggerEx.MDCCloseable
        pass

    def mdcRemove(self, key):
        # type: (String) -> None
        pass

    def mdcSet(self):
        # type: () -> LoggerEx.MDCCloseable
        pass

    @staticmethod
    def newBuilder():
        # type: () -> LoggerEx.Builder
        pass

    def setIdentObject(self, identObj):
        # type: (Object) -> None
        pass

    def setToStringStyle(self, toStringStyle):
        # type: (ToStringStyle) -> None
        pass

    def trace(self, message, t=None):
        # type: (String, Optional[Throwable]) -> None
        pass

    def traceDuration(self, message):
        # type: (String) -> Closeable
        pass

    def traceEvent(self, message, *args):
        # type: (String, Any) -> None
        pass

    def tracef(self, message, *args):
        # type: (String, Any) -> None
        pass

    def warn(self, message, t=None):
        # type: (String, Optional[Throwable]) -> None
        pass

    def warnEvent(self, message, *args):
        # type: (String, Any) -> None
        pass

    def warnf(self, message, *args):
        # type: (String, Any) -> None
        pass

    class Builder(Object):
        def build(self, *args):
            # type: (Any) -> LoggerEx
            pass

        def eventSystem(self, systemId):
            # type: (str) -> LoggerEx.Builder
            pass

        def identObject(self, identObj):
            # type: (Object) -> LoggerEx.Builder
            pass

        def mdcContext(self, *args):
            # type: (Object) -> LoggerEx.Builder
            pass

        def mutableIdentObject(self, identObj):
            # type: (Object) -> LoggerEx.Builder
            pass

    class MDCCloseable(Object, AutoCloseable):
        def close(self):
            # type: () -> None
            pass



"""Twilio Functions.

The following functions give you access to read info and send SMS
through Twilio. This requires the Twilio Module, which is not included
in a typical install.
"""

from __future__ import print_function

__all__ = [
    "getAccounts",
    "getAccountsDataset",
    "getPhoneNumbers",
    "getPhoneNumbersDataset",
    "sendSms",
]

from typing import List

from com.inductiveautomation.ignition.common import BasicDataset
from java.lang import String


def getAccounts():
    # type: () -> List[String]
    """Return a list of Twilio accounts that have been configured in the
    Gateway.

    Returns:
        A list of configured Twilio accounts.
    """
    return ["twilio_account1", "twilio_account2"]


def getAccountsDataset():
    # type: () -> BasicDataset
    """Return a list of Twilio accounts that have been configured in the
    Gateway as a single-column Dataset.

    Returns:
        A list of configured Twilio accounts as a single-column Dataset.
    """
    return BasicDataset()


def getPhoneNumbers(accountName):
    # type: (String) -> List[String]
    """Returns a list of outgoing phone numbers for a Twilio account.

    Note that these numbers are supplied by Twilio, and are not defined
    on a user in Ignition.

    Args:
        accountName: The Twilio account to retrieve phone numbers for.

    Returns:
        A list of phone numbers for the given Twilio account.
    """
    phoneNumbers = []  # type: List[String]
    if accountName == "Jenny":
        phoneNumbers.append("+12058675309")
    return phoneNumbers


def getPhoneNumbersDataset(accountName):
    # type: (String) -> BasicDataset
    """Return a list of outgoing phone numbers for a Twilio account as a
    single-column Dataset.

    Note that these numbers are supplied by Twilio, and are not defined
    on a user in Ignition.

    Args:
        accountName: The Twilio account to retrieve phone numbers for.

    Returns:
        A list of phone numbers for the given Twilio account as a
        single-column Dataset.
    """
    print(accountName)
    return BasicDataset()


def sendSms(accountName, fromNumber, toNumber, message):
    # type: (String, String, String, String) -> None
    """Sends an SMS message.

    Args:
        accountName: The Twilio account to send the SMS from.
        fromNumber: The outbound phone number belonging to the Twilio
            account to use.
        toNumber: The phone number of the recipient.
        message: The body of the SMS.
    """
    print(accountName, fromNumber, toNumber, message)



__all__ = ["LocalizedString"]

from typing import Any, List, Optional

from java.lang import Object, String
from java.util import Locale


class LocalizedString(Object):
    def __init__(self, *args):
        # type: (Any) -> None
        pass

    def compareTo(self, other):
        # type: (LocalizedString) -> int
        pass

    @staticmethod
    def createRaw(stringVal):
        # type: (String) -> LocalizedString
        pass

    def getDefaultVal(self):
        # type: () -> String
        pass

    def getKey(self):
        # type: () -> String
        pass

    def getParams(self):
        # type: () -> List[Object]
        pass

    def setDefaultVal(self, defaultVal):
        # type: (String) -> None
        pass

    def setKey(self, key):
        # type: (String) -> None
        pass

    def setParams(self, params):
        # type: (List[Object]) -> None
        pass

    def toString(self, locale=None):
        # type: (Optional[Locale]) -> String
        pass



from __future__ import print_function

__all__ = ["AbstractOPCUtilities", "DatasetUtilities", "SProcCall", "SystemUtilities"]

from typing import Any, Dict, List, Optional, Union

from com.inductiveautomation.ignition.common import BasicDataset, Dataset
from com.inductiveautomation.ignition.common.model.values import QualityCode
from com.inductiveautomation.ignition.common.opc import BrowseElementType
from com.inductiveautomation.ignition.common.script.abc import AbstractJythonSequence
from com.inductiveautomation.ignition.common.script.message import Request
from java.lang import Class, Object, String
from java.util import Locale
from org.python.core import PyObject


class AbstractOPCUtilities(Object):
    def browseServer(self, opcServer, nodeId):
        # type: (String, String) -> List[AbstractOPCUtilities.PyOPCTag]
        return [
            AbstractOPCUtilities.PyOPCTag(opcServer, nodeId, "", BrowseElementType())
        ]

    def getServers(self):
        pass

    def getServerState(self, opcServer):
        pass

    def isServerEnabled(self, serverName):
        pass

    def readValue(self, opcServer, itemPath):
        pass

    def readValues(self, opcServer, itemPaths):
        pass

    def setServerEnabled(self, serverName, enabled):
        pass

    def writeValue(self, *args, **kwargs):
        pass

    def writeValues(self, *args, **kwargs):
        pass

    class PyOPCTag(PyObject):
        displayName = None  # type: String
        elementType = None  # type: BrowseElementType
        nodeId = None  # type: String
        serverName = None  # type: String

        def __init__(self, serverName, nodeId, displayName, elementType):
            # type: (String, String, String, BrowseElementType) -> None
            self.serverName = serverName
            self.nodeId = nodeId
            self.displayName = displayName
            self.elementType = elementType
            super(AbstractOPCUtilities.PyOPCTag, self).__init__()

        def __findattr_ex__(self, name):
            # type: (str) -> PyObject
            pass

        def getDisplayName(self):
            # type: () -> String
            return self.displayName

        def getElementType(self):
            # type: () -> BrowseElementType
            return self.elementType

        def getNodeId(self):
            # type: () -> String
            return self.nodeId

        def getServerName(self):
            # type: () -> String
            return self.serverName


class DatasetUtilities(Object):
    @staticmethod
    def addColumn(*args):
        pass

    @staticmethod
    def addRow(*args):
        pass

    @staticmethod
    def addRows(*args):
        pass

    @staticmethod
    def appendDataset(ds1, ds2):
        pass

    @staticmethod
    def clearDataset(ds):
        pass

    @staticmethod
    def dataSetToExcel(headerRow, datasets):
        pass

    @staticmethod
    def dataSetToExcelBytes(headerRow, objects, nullsEmpty, sheetNames):
        pass

    @staticmethod
    def dataSetToExcelStreaming(headerRow, objects, out, nullsEmpty):
        pass

    @staticmethod
    def dataSetToHTML(headerRow, ds, title):
        pass

    @staticmethod
    def dataSetToHTMLStreaming(headerRow, ds, title, fw):
        pass

    @staticmethod
    def deleteRow(ds, row):
        pass

    @staticmethod
    def deleteRows(ds, rows):
        pass

    @staticmethod
    def filterColumns(dataset, columns):
        pass

    @staticmethod
    def formatDates(dataset, format, locale=Locale.US):
        pass

    @staticmethod
    def fromCSV(csv):
        pass

    @staticmethod
    def fromCSVJava(csv):
        pass

    @staticmethod
    def getColumnHeaders(ds):
        pass

    @staticmethod
    def insertColumn(*args):
        pass

    @staticmethod
    def insertRow(*args):
        pass

    @staticmethod
    def setValue(*args):
        pass

    @staticmethod
    def sort(
        ds,  # type: BasicDataset
        keyColumn,  # type: Union[int, str]
        ascending=None,  # type: Optional[bool]
        naturalOrdering=None,  # type: Optional[bool]
    ):
        # type: (...) -> BasicDataset
        pass

    @staticmethod
    def toCSV(*args, **kwargs):
        pass

    @staticmethod
    def toCSVJava(ds, showHeaders, forExport, localized=False):
        pass

    @staticmethod
    def toCSVJavaStreaming(ds, showHeaders, forExport, sw, localized):
        pass

    @staticmethod
    def toDataSet(*args):
        pass

    @staticmethod
    def toExcel(*args, **kwargs):
        pass

    @staticmethod
    def toJSONObject(data):
        pass

    @staticmethod
    def toPyDataSet(dataset):
        # type: (Dataset) -> PyDataSet
        pass

    @staticmethod
    def updateRow(ds, row, changes):
        pass

    class PyDataSet(Dataset, AbstractJythonSequence):
        def __init__(self, ds=None):
            # type: (Optional[Dataset]) -> None
            pass

        def getColumnCount(self):
            # type: () -> int
            """Returns the number of columns in the dataset.

            Returns:
                The number of columns in the dataset.
            """
            pass

        def getColumnIndex(self, colName):
            # type: (String) -> int
            """Returns the index of the column with the name colName.

            Args:
                colName: The name of the column.

            Returns:
                The index of the column with the name colName.
            """
            pass

        def getColumnName(self, col):
            # type: (int) -> String
            """Returns the name of the column at the index colIndex.

            Args:
                col: The column number. Zero-indexed.

            Returns:
                The name of the column at the index colIndex.
            """
            pass

        def getColumnNames(self):
            # type: () -> List[String]
            """Returns a list with the names of all the columns.

            Returns:
                A list with the names of all the columns.
            """
            pass

        def getColumnType(self, col):
            # type: (int) -> Class
            """Returns the type of the column at the index.

            Args:
                col: The column number. Zero-indexed.

            Returns:
                The type of the column at the index.
            """
            pass

        def getColumnTypes(self):
            # type: () -> List[Class]
            """Returns a list with the types of all the columns.

            Returns:
                A list with the types of all the columns.
            """
            pass

        def getPrimitiveValueAt(self, row, col):
            # type: (int, int) -> float
            """If the given column is a numeric type or a Date, then the
            value will be returned as a double.

            Args:
                row: The row index. Zero-based index.
                col: The column index. Zero-based index.

            Raises:
                IllegalArgumentException: if the value at row, col is
                    not a number or Date.
                UnsupportedOperationException: If the Dataset
                    implementation declines to implement this operation.
            """
            pass

        def getQualityAt(self, row, col):
            # type: (int, int) -> QualityCode
            """Returns the quality of the value at the given location.

            Args:
                row: The row index. Zero-based index.
                col: The column index. Zero-based index.

            Raises:
                ArrayIndexOutOfBoundsException: If the given row, col is
                    out of range and hasQualityData() returns true.
            """
            pass

        def getRowCount(self):
            # type: () -> int
            """Returns the number of rows in the dataset.

            Returns:
                The number of rows in the dataset.
            """
            pass

        def getValueAt(self, row, col):
            # type: (int, Union[int, String]) -> Any
            """Returns the value at the specified row index and column
            name or index.

            Args:
                row: The row number. Zero-indexed.
                col: The column number (zero-indexed) or name.

            Returns:
                The value found at the row and column.
            """
            pass

        def setData(self, data):
            # type: (Dataset) -> None
            """Used for serialization only."""
            pass


class SProcCall(Object):
    callFinished = False  # type: bool
    datasource = ""  # type: String
    params = None  # type: Dict[SProcCall.SProcArgKey, SProcCall.SProcArg]
    procedureName = None  # type: String
    resultset = None  # type: Dataset
    returnParam = None  # type: SProcCall.SProcArg
    skipAudit = None  # type: bool
    txId = None  # type: String
    updateCount = None  # type: int

    def getDataSource(self):
        # type: () -> str
        pass

    def getOutParamValue(self, param):
        # type: (Union[int, str]) -> Any
        print(self, param)
        return 0

    def getProcedureName(self):
        # type: () -> str
        pass

    def getResultSet(self):
        # type: () -> BasicDataset
        print(self)
        return BasicDataset()

    def getReturnValue(self):
        # type: () -> Any
        print(self)
        return 0

    def getTxId(self):
        # type: () -> str
        return "transaction_id"

    def getUpdateCount(self):
        # type: () -> int
        print(self)
        return 1

    def isSkipAudit(self):
        # type: () -> bool
        print(self)
        return False

    def registerInParam(self, param, typeCode, value):
        # type: (Union[int, str], int, Any) -> None
        print(self, param, typeCode, value)

    def registerOutParam(self, param, typeCode):
        # type: (Union[int, str], int) -> None
        print(self, param, typeCode)

    def registerReturnParam(self, typeCode):
        # type: (int) -> None
        print(self, typeCode)

    def setSkipAudit(self, skipAudit):
        # type: (bool) -> None
        pass

    def setTxId(self, txId):
        # type: (String) -> None
        pass

    class SProcArg(Object):
        outParam = False  # type: bool
        inParam = False  # type: bool
        paramType = None  # type: int
        value = None  # type: Object

        def getParamType(self):
            # type: () -> int
            pass

        def getValue(self):
            # type: () -> Any
            pass

        def isInParam(self):
            pass

        def isOutParam(self):
            pass

        def setParamType(self, paramType):
            pass

        def setValue(self, value):
            pass

    class SProcArgKey(Object):
        index = -1  # type: int
        name = None  # type: String

        def getParamIndex(self):
            pass

        def getParamName(self):
            pass

        def isNamedParam(self):
            pass


class SystemUtilities(Object):
    @staticmethod
    def logger(loggerName):
        pass

    @staticmethod
    def parseTranslateArguments(*args, **kwargs):
        pass

    class RequestImpl(Object, Request):
        timeout = None  # type: int

        def __init__(self, timeout):
            # type: (int) -> None
            self.timeout = timeout

        def checkTimeout(self):
            pass

        def dispatchFunc(self):
            pass

        def finishExceptionally(self, e):
            pass

        def finishSuccessfully(self, value):
            pass

        def getLongId(self):
            pass

        def cancel(self):
            pass

        def get(self):
            pass

        def getError(self):
            pass

        def onError(self, func):
            pass

        def onSuccess(self, func):
            pass
